# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# Authors: Michal Minar <miminar@redhat.com>
# -*- coding: utf-8 -*-
"""
    .. autoclass:: IndicationManager
        :members:
"""

import pywbem
from Queue import Queue
import re
import socket
import threading

from lmi.base import singletonmixin
from lmi.providers import cmpi_logging

RE_FILTER_NAME = re.compile(r'^(?P<prefix>lmi:'
    r'(?P<class_name>[a-z0-9_]+):)(?P<filter_id>.*)$', re.IGNORECASE)

FILTER_DEFAULTS = {
        "SourceNamespace"  : "root/cimv2",
        "SourceNamespaces" : ["root/cimv2"],
        "QueryLanguage"    : "CIM:CQL"
}

LOG = cmpi_logging.get_logger(__name__)

@cmpi_logging.trace_function
def enumerate_namespaces(ch):
    """
    Return tuple ``([CIM_Namespace instance, ...], ns_interop)``. Where
    first item is a list of object paths of all namespaces in broker and
    the second is a name of namespace, where this information can be found.

    :param ch: CIMOM handle.
    """
    nsclasses = ["CIM_Namespace", "__Namespace"]
    namespaces = ['root/cimv2', 'root/PG_InterOp', 'Interop',
            'interop', 'root', 'root/interop']
    nspaths = []
    ns = None
    for cls in nsclasses:
        for ns in namespaces:
            try:
                nspaths = [nm for nm in ch.EnumerateInstanceNames(ns, cls)]
                if nspaths:
                    break
            except pywbem.CIMError as exc:
                if exc[0] in (
                        pywbem.CIM_ERR_INVALID_NAMESPACE,
                        pywbem.CIM_ERR_NOT_SUPPORTED,
                        pywbem.CIM_ERR_INVALID_CLASS):
                    pass
                if exc[0] == pywbem.CIM_ERR_FAILED:
                    LOG().error("EnumerateInstanceNames failed for %s:%s: %s",
                            ns, cls, str(exc))
                else:
                    raise
        if nspaths:
            break
    if not nspaths:
        LOG().error("failed to enumerate namespaces")
        ns = None
    return (nspaths, ns)

@cmpi_logging.trace_function
def find_ns_interop(ch):
    """
    Return name of interop namespace, where ``CIM_IndicationFilter``
    class reside.

    :param ch: CIMOM handle.
    """
    _, ns_interop = enumerate_namespaces(ch)
    return ns_interop

def make_filter_name(class_name, fltr_id):
    """
    Return value for ``CIM_IndicationFilter.Name`` property.
    """
    return "LMI:%s:%s" % (class_name, fltr_id)

def parse_filter_name(name):
    """
    Return tuple ``(class_name, filter_id)``.

    :param name: (``string``) Value of cim filter's *Name* property.
    """
    match = RE_FILTER_NAME.match(name)
    if not match:
        raise ValueError('Could not parse filter name: "%s"' % name)
    return (match.group("class_name"), match.group("filter_id"))

@cmpi_logging.trace_function
def make_indication_filter_path(class_name, fltr_id, ns_interop):
    """
    Return CIM_IndicationFilter instance path for given filter id.

    :param class_name: (``string``) *Scoped class* name.
    :param fltr_id: (``string``) Filter name.
    :param ns_interop: (``string``) Interop namespace.
    """
    for arg in ('class_name', 'fltr_id', 'ns_interop'):
        if not isinstance(locals()[arg], basestring):
            raise TypeError("%s must be basestring" % arg)
    cop = pywbem.CIMInstanceName("CIM_IndicationFilter",
            namespace=ns_interop)
    cop['CreationClassName'] = 'CIM_IndicationFilter'
    cop['SystemCreationClassName'] = 'CIM_ComputerSystem'
    cop['SystemName'] = socket.gethostname()
    cop['Name'] = make_filter_name(class_name, fltr_id)
    return cop

@cmpi_logging.trace_function
def remove_cimom_filter(ch, fltr_path):
    """
    Deletes instance of CIM_IndicationFilter installed at broker with all
    referencing subscriptions.

    Returns list of subscription instace names, that were deleted.

    :param ch: CIMOM handle.
    :param fltr_path: (``CIMInstanceName``) Path of ``CIM_IndicationFilter`` to
        remove.
    """
    if not isinstance(fltr_path, pywbem.CIMInstanceName):
        raise TypeError("fltr_path must be a CIMInstanceName")

    referents = []
    for ref in ch.AssociatorNames(fltr_path,
            role="Filter",
            resultRole="Handler",
            resultClass="CIM_IndicationSubscription"):
        ch.DeleteInstance(ref)
        referents.append(ref)
    ch.DeleteInstance(fltr_path)
    LOG().debug('removed indication filter "%s" with %d referents',
            fltr_path["Name"], len(referents))
    return referents

class IndicationManager(singletonmixin.Singleton):
    """
    Using ``IndicationManager`` class
    providers can send indications without bothering with handling of
    indication subscriptions.

    Usage:

    1. Subclass CIM_InstCreation and CIM_InstModification.

    2. In your initialization routine, create one ``IndicationManager``
    instance. E.g. one for whole ``LMI_Storage`` may is enough. Like
    this::

        indication_manager = \
                IndicationManager.get_instance(env, "Storage", "root/cimv2")

    3. Call ``indication_manager.add_filters()`` with all filters your
    providers support for particular CIM class. This method can be called
    multiple times.
    For example::

      filters = {
          "JobPercentUpdated": {
              "Query" : "SELECT * FROM CIM_InstModification WHERE"
                " SourceInstance ISA LMI_StorageJob AND"
                " SourceInstance.CIM_ConcreteJob::PercentComplete <>"
                " PreviousInstance.CIM_ConcreteJob::PercentComplete",
              "Description" : "Modification of Percentage Complete for"
                " a Concrete Job.",
          },
          "JobSucceeded": {
              "Query" : "SELECT * FROM CIM_InstModification WHERE"
                " SourceInstance ISA LMI_StorageJob AND"
                " SourceInstance.CIM_ConcreteJob::JobState = "
                " CIM_ConcreteJob.JobState#'Completed'",
              "Description": "Modification of Operational Status for"
                " a Concrete Job to 'Complete' and 'OK'.",
          },
          #... other indications
      }
      instance_manager.add_filters("LMI_StorageJob", filters)

    First argument is a name of class to which indications apply. We'll call
    it *Scoping class*.

    4. In your provider module, implement indication functions like this::

        def authorize_filter(env, fltr, ns, classes, owner):
            indication_manager.authorize_filter(env, fltr, ns, classes, owner)

        def activate_filter (env, fltr, ns, classes, first_activation):
            indication_manager.activate_filter(env, fltr, ns, classes,
                first_activation)

        def deactivate_filter(env, fltr, ns, classes, last_activation):
            indication_manager.deactivate_filter(env, fltr, ns, classes,
                last_activation)

        def enable_indications(env):
            indication_manager.enable_indications(env)

        def disable_indications(env):
            indication_manager.disable_indications(env)

    From now on, the ``IndicationManager`` will track all subscribed filters.
    You can query the ``indication_manager.is_subscribed()`` before you create
    and send an indication. Use ``indication_manager.send_indication()``
    to send your indications.

    Only static (=preconfigured, read-only) indication filters are
    supported.

    For user to use these preconfigured filters, they need to be installed
    at broker as instances of ``CIM_IndicationFilter``. But since they can
    not be guarded against removel by accident, this object provides a way
    to reinstall them. But using this is not recomended, since it can upset
    users. See :ref:`_update_context-label`.

    The supported filters must be passed to add_filters method. The filters
    are passed as dictionary ``'filter_id' -> {dictionary 'IndicationFilter
    property' -> 'value'}``. There must be at least ``Query`` property in
    each filter, CQL is assumed.

    This helper automatically tracks which filters are subscribed. Provider
    can query ``is_subscribed()`` to check, if filter with given
    ``filter_id`` is subscribed before generating indications.

    The CMPI interface to send indications is complicated -
    when an indication is send from CIMOM callback (e.g. ``get_instance``),
    it must use current ``env`` parameter of the callback and it would be
    tedious to pass it to ``IndicationManager`` each time. Therefore
    ``IndicationManager`` creates its own thread, registers it at CIMOM
    using ``PrepareAttachThread``/``AttachThread``.

    As side-effect, indication can be sent from any thread, there is no
    need to call ``PrepareAttachThread``/``AttachThread``.
    """
    SEVERITY_INFO = pywbem.Uint16(2)  # CIM_Indication.PerceivedSeverity

    COMMAND_STOP = 1  # Command to the IndicationManager thread to stop.

    @cmpi_logging.trace_method
    def __init__(self, env, nameprefix, namespace, ns_interop=None,
            queue=None):
        """
        Create new ``IndicationManager``. Usually only one instance
        is necessary for one provider process.

        :param env: (``ProviderEnvironment``) Provider enviroment, taken
            from CIMOM callback (e.g. ``get_providers()``).
        :param nameprefix: (``string``) Prefix of your ``CIM_InstCreation``
            and ``CIM_InstModification`` subclasses, e.g. 'Storage' for
            ``LMI_StorageInstCreation``.
        :param namespace: (``string``) Namespace, which will be set to
            outgoing indications instances.
        :param ns_interop: (``string``) Namespace, where filters and
            subscriptions are stored.
        :param queue: Optional custom input queue with the same interface as
            ``Queue.Queue``.
        """

        # { class_name :
        #   { filter_id : filter_properties
        #   , ... }
        # }
        self._filters = pywbem.NocaseDict()
        self._enabled = False
        # { (class_name, filter_id), ... }
        self._subscribed_filters = set()
        self._nameprefix = nameprefix
        self._namespace = namespace
        self._ns_interop = ns_interop
        self._access_lock = threading.RLock()
        self._env = env

        if queue is None:
            queue = Queue()
        self._queue = queue
        # prepare indication thread
        ch = env.get_cimom_handle()
        new_broker = ch.PrepareAttachThread()
        self._indication_sender = threading.Thread(
                target=self._send_indications_loop, args=(new_broker,))
        self._indication_sender.daemon = False
        self._indication_sender.start()

    @property
    def enabled(self):
        """
        Return a boolean saying, whether indication sending is enabled.
        """
        with self._access_lock:
            return self.enabled

    @property
    def namespace(self):
        """
        Return namespace of outgoing indication instances.
        """
        return self._namespace

    @property
    def nameprefix(self):
        """
        Return prefix of indication class names.
        """
        return self._nameprefix

    @property
    def ns_interop(self):
        """
        Return interop namespace name.
        """
        with self._access_lock:
            if self._ns_interop is None:
                ch = self._env.get_cimom_handle()
                self._ns_interop = find_ns_interop(ch)
                LOG().info('found interop namespace: %s', self._ns_interop)
            return self._ns_interop

    @property
    def instcreation_classname(self):
        """
        Return whole class name of InstCreation indication.
        """
        return "LMI_" + self._nameprefix + "InstCreation"

    @property
    def instmodification_classname(self):
        """
        Return whole class name of InstModification indication.
        """
        return "LMI_" + self._nameprefix + "InstModification"

    @property
    def instdeletetion_classname(self):
        """
        Return whole class name of InstDeletion indication.
        """
        return "LMI_" + self._nameprefix + "InstDeletion"

    @cmpi_logging.trace_method
    def _get_filter_inst(self, class_name, fltr_id):
        """
        Return instance of CIM_IndicationFilter registered in CIMOM if any.

        :param class_name: (``string``) *Scoping class* name.
        :param fltr_id: (``string``) Indication name.
        """
        ch = self._env.get_cimom_handle()
        cop = make_indication_filter_path(class_name, fltr_id, self.ns_interop)
        try:
            return ch.GetInstance(cop)
        except pywbem.CIMError as exc:
            if exc.args[0] == pywbem.CIM_ERR_NOT_FOUND:
                return None
            raise

    @cmpi_logging.trace_method
    def _ensure_cimom_has_filter(self, class_name, fltr_id):
        """
        Ensures, that cimom has ``fltr_id`` filter registered as instance.
        If it has, but the query differs it is recreated at broker.

        :param class_name: (``string``) *Scoping class* name.
        :param fltr_id: (``string``) Indication name.
        """
        inst = self._get_filter_inst(class_name, fltr_id)
        ch = self._env.get_cimom_handle()
        installed = inst is not None
        referents = []
        if installed:
            for prop_name, val in self._filters[class_name][fltr_id].items():
                if inst[prop_name] != val:
                    LOG().info("filter \"%s\" is installed, but its property"
                            " \"%s\" has outdated value; removing...",
                            fltr_id, prop_name)
                    referents = remove_cimom_filter(ch, inst.path)
                    installed = False
        if not installed:
            if inst is not None:
                path = inst.path
            else:
                path = make_indication_filter_path(class_name, fltr_id,
                        self.ns_interop)
            inst = pywbem.CIMInstance(path.classname, path=path)
            kwargs = FILTER_DEFAULTS.copy()
            for key, val in path.keybindings.items():
                kwargs[key] = val
            kwargs.update(self._filters[class_name][fltr_id])
            inst.update(kwargs)
            try:
                inst = ch.CreateInstance(inst)
                LOG().info("filter \"%s\" installed", fltr_id)
            except pywbem.CIMError:
                LOG().exception("failed to install indication filter \"%s\"",
                        fltr_id)
            if referents:
                LOG().debug('reinstalling %d filter subscriptions',
                        len(referents))
                for ref in referents:
                    ch.CreateInstance(ref)
        return inst

    @cmpi_logging.trace_method
    def _get_matching_filter(self, query):
        """
        Try to find matching filter properties in local ``_filters`` storage
        and return it. ``None`` is returned if not found.

        Return a tuple ``(class_name, filter_id, filter_properties)``.

        :param query: (``string``) Is filter query.
        """
        if not isinstance(query, basestring):
            raise TypeError("query must be a string")
        for clsname, fltrs in self._filters.iteritems():
            for fltr_id, props in fltrs.iteritems():
                if query == props["Query"]:
                    return (clsname, fltr_id, props)
        return None

    @cmpi_logging.trace_method
    def ensure_filters_installed(self, class_name=None, fltr_id=None):
        """
        This function checks for existence of filters at broker. Filters
        must be registered with this instance before the check can be done.
        Without arguments all registered filters will be checked.
        
        :param class_name: (``string``) Name of *Scoped class* that reduces
            searched filters.
        :param fltr_id: (``string``) Indication name reducing filters that
            will be checked.
        """
        cls_to_check = self._filters.keys()
        if class_name is not None:
            cls_to_check = [class_name]
        filters_to_check = list(
                    (c, f)
                for c in cls_to_check
                for f in self._filters[c].keys()
                if fltr_id is None or fltr_id == f)
        with self._access_lock:
            try:
                for clsname, fltr_id in filters_to_check:
                    self._ensure_cimom_has_filter(clsname, fltr_id)
                LOG().debug('filters installed')
                return True
            except pywbem.CIMError as exc:
                if exc.args[0] == pywbem.CIM_ERR_ACCESS_DENIED:
                    LOG().error("filters could not be checked"
                            " for presence due to invalid context")
                    return False
                raise

    @cmpi_logging.trace_method
    def update_context(self, env):
        """
        .. _update_context-label

        When ``IndicationManager`` is initialized upon provider initialization,
        the conxet given does not contain any user credentials that are
        needed for communication with broker. In order to check for filter's
        existence at broker, this method needs to be called first with
        context containing user's credentials.

        This needs to be called only once.

        **Note** that if you don't plan to check for filter's presence at
        broker at runtime, you are not interested in this function.
        """
        with self._access_lock:
            self._env = env

    @cmpi_logging.trace_method
    def add_filters(self, class_name, filters, ensure_installed=False):
        """
        Add new filters to the helper. These filters will be allowed for
        subscription.

        :param filters: (``dictionary filter_id -> filter properties``)
            The filters. ``filter properties`` is dictionary
            ``property_name -> value``, where at least ``Query`` property
            must be set. ``Name`` property will be automatically created
            as 'LMI:<class_name>:<filter_id>'.
        :param ensure_installed: (``bool``) Whether to check for filter presence
            at broker and install them if missing. **Note** That in order
            for this to work, the context must be updated with user's
            credentials. See :ref:`update_context-label`.
        """
        with self._access_lock:
            if not class_name in self._filters:
                self._filters[class_name] = pywbem.NocaseDict()
            self._filters[class_name].update(filters)
            if ensure_installed:
                self.ensure_filters_installed(class_name=class_name)

    @cmpi_logging.trace_method
    def authorize_filter(self, _env, fltr, _class_name, _op, _owner):
        """
        AuthorizeFilter callback from CIMOM. Call this method from appropriate
        CIMOM callback

        It asks us to verify whether this filter is allowed.

        :param fltr: Contains the filter that must be authorized. 
        :param _class_name: (``String``) Contains the class name extracted
            from the filter FROM clause. 
        :param _op: The name of the class for which monitoring is required.
            Only the namespace part is set if className is a process indication.
        :param _owner The owner argument is the destination owner. 
        """
        with self._access_lock:
            res = self._get_matching_filter(fltr)
            if res is not None:
                self._subscribed_filters.add((res[0], res[1]))
                LOG().info("InstanceFilter %s: %s authorized",
                        make_filter_name(res[0], res[1]), fltr)
                return True
            return False

    @cmpi_logging.trace_method
    def activate_filter(self, _env, fltr, _class_name, _class_path,
            first_activation):
        """
        ActivateFilter callback from CIMOM. Call this method from appropriate
        CIMOM callback.

        It ask us to begin monitoring a resource. The function shall begin
        monitoring the resource according to the filter express only.

        :param fltr: The filter argument contains the filter specification
            for this subscription to become active. 
        :param _class_name: (``String``) The class name extracted from the filter
            FROM clause. 
        :param _class_path: (``CIMInstanceName``) The name of the class for
            which monitoring is required. Only the namespace part is set if
            eventType is a process indication. 
        :param first_activation: (``bool``) Set to true if this is the first
            filter for className. 
        """
        with self._access_lock:
            if not first_activation:
                return
            res = self._get_matching_filter(fltr)
            if res is not None:
                self._subscribed_filters.add((res[0], res[1]))
                LOG().info("InstanceFilter %s: %s started",
                        make_filter_name(res[0], res[1]), fltr)

    @cmpi_logging.trace_method
    def deactivate_filter(self, _env, fltr, _class_name, _class_path,
            last_activation):
        """
        DeactivateFilter callback from CIMOM. Call this method from appropriate
        CIMOM callback.

        Informs us that monitoring using this filter should stop.

        :param fltr: The filter argument contains the filter specification for
            this subscription to become active. 
        :param class_name: (``String``) The class name extracted from the filter
            FROM clause. 
        :param class_path: (``CIMInstanceName``) class_path The name of the
            class for which monitoring is required. Only the namespace part is
            set if className is a process indication. 
        :last_activation: (``bool``) Set to true if this is the last filter for
            className. 
        """
        with self._access_lock:
            if not last_activation:
                return
            res = self._get_matching_filter(fltr)
            if res is not None:
                self._subscribed_filters.remove((res[0], res[1]))
                LOG().info("InstanceFilter %s: %s stopped",
                        make_filter_name(res[0], res[1]), fltr)

    @cmpi_logging.trace_method
    def enable_indications(self, _env):
        """
        EnableIndications callback from CIMOM. Call this method from
        appropriate CIMOM callback.

        Tells us that indications can now be generated. The MB is now prepared
        to process indications. The function is normally called by the MB after
        having done its intialization and processing of persistent subscription
        requests.
        """
        with self._access_lock:
            self._enabled = True
        LOG().info("Indications enabled")

    @cmpi_logging.trace_method
    def disable_indications(self, _env):
        """
        EnableIndications callback from CIMOM. Call this method from
        appropriate CIMOM callback.

        Tells us that we should stop generating indications. MB will not accept
        any indications until enabled again. The function is normally called
        when the MB is shutting down indication services either temporarily or
        permanently.
        """
        with self._access_lock:
            self._enabled = False
        LOG().info("Indications disabled")

    @cmpi_logging.trace_method
    def send_indication(self, indication):
        """
        Send indication to all subscribers. Call this method from appropriate
        CIMOM callback.
        """
        self._queue.put(indication)

    @cmpi_logging.trace_method
    def send_instcreation(self, instance, filter_id):
        """
        Send ``LMI_<nameprefix>InstCreation`` indication with given instance.

        :param instance: (``CIMInstance``) The created instance.
        :param filter_id: (``string``) The ID of registered filter which
            corresponds to this indication.
        """
        if not self.is_subscribed(instance.classname, filter_id):
            return
        path = pywbem.CIMInstanceName(
                classname=self.instcreation_classname,
                namespace=self.namespace)
        ind = pywbem.CIMInstance(
                self.instcreation_classname,
                path=path)
        ind['SourceInstance'] = instance
        ind['SourceInstanceHost'] = socket.gethostname()
        ind['SourceInstanceModelPath'] = str(instance.path)
        ind['IndicationFilterName'] = make_filter_name(
                instance.classname, filter_id)
        ind['PerceivedSeverity'] = self.SEVERITY_INFO

        LOG().info("Sending indication %s for %s",
                ind["IndicationFilterName"], str(path))
        self.send_indication(ind)

    @cmpi_logging.trace_method
    def send_instmodification(self, old_instance, new_instance, filter_id):
        """
        Send ``LMI_<nameprefix>InstModification`` indication with given
        instance.

        :param old_instance: (``CIMInstance``) The instance before
            modification.
        :param new_instance: (``CIMInstance``) The instance after modification.
        :param filter_id: (``string``) The ID of registered filter which
            corresponds to this indication.
        """
        if not self.is_subscribed(new_instance.classname, filter_id):
            return
        path = pywbem.CIMInstanceName(
                classname=self.instmodification_classname,
                namespace=self.namespace)
        ind = pywbem.CIMInstance(
                self.instcreation_classname,
                path=path)
        ind['SourceInstance'] = new_instance
        ind['PreviousInstance'] = old_instance
        ind['SourceInstanceHost'] = socket.gethostname()
        ind['SourceInstanceModelPath'] = str(new_instance.path)
        ind['IndicationFilterName'] = make_filter_name(
                new_instance.classname, filter_id)
        ind['PerceivedSeverity'] = self.SEVERITY_INFO

        LOG().info("Sending indication %s for %s",
                ind["IndicationFilterName"], str(path))
        self.send_indication(ind)

    @cmpi_logging.trace_method
    def is_subscribed(self, class_name, fltr_id):
        """
        Return True, if there is someone subscribed for given filter.

        :param class_name: (``string``) *Scoping class* name.
        :param fltr_id: (``string``) ID of the filter to check.
        """
        with self._access_lock:
            if not self._enabled:
                return False
            if (class_name, fltr_id) in self._subscribed_filters:
                return True
            return False

    @cmpi_logging.trace_method
    def is_registered(self, class_name, fltr_id):
        """
        Return True, if filter id has been registered with current instance.

        :param class_name: (``string``) *Scoping class* name.
        :param fltr_id: (``string``) ID of the filter to check.
        """
        with self._access_lock:
            return (class_name in self._filters
                   and fltr_id in self._filters[class_name])

    def _send_indications_loop(self, broker):
        """
        This method runs in its own thread. It just sends all enqueued
        indications.

        :param broker: (``BrokerCIMOMHandle``) Handle of the CIMOM.
        """
        broker.AttachThread()
        while True:
            command = self._queue.get()

            if isinstance(command, pywbem.CIMInstance) :
                indication = command
                LOG().trace_info("Delivering indication %s",
                        str(indication.path))
                broker.DeliverIndication(self.namespace, indication)

            elif isinstance(command, int):
                LOG().trace_info("Received command %d", command)
                if command == self.COMMAND_STOP:
                    if hasattr(self._queue, "task_done"):
                        self._queue.task_done()
                    break

            if hasattr(self._queue, "task_done"):
                self._queue.task_done()

        LOG().info("Stopped Indication thread.")

    @cmpi_logging.trace_method
    def shutdown(self):
        """
        Stop the thread. This method blocks until the thread is safely
        destroyed.
        """
        self._queue.put(self.COMMAND_STOP)
        self._indication_sender.join()



# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
This module defines session manager.
"""
import threading
import Queue as TQueue   # T as threaded
from multiprocessing import Queue

from lmi.providers import cmpi_logging
from lmi.software.util import Configuration
from lmi.software.util import get_signal_name
from lmi.software.yumdb import errors
from lmi.software.yumdb import jobs
from lmi.software.yumdb.process import YumWorker

#: Number of times the worker process will be resurrected for a completion of
#: single job. If the process dies afterwards (while still doing the same job)
#: an exception will be raised.
MAX_RESURRECTIONS = 1

LOG = cmpi_logging.get_logger(__name__)

class SessionManager(threading.Thread):
    """
    It manages *YumWorker* process that handles synchronous and asynchornous
    jobs. When this processed is killed it will be respawned, and currently
    handled job will be redone until finished. Session manager allows to
    create sessions. They cause *YumWorker* to lock the database for exclusive
    use until the session is over.

    Just one job may be processed at a time. When the job is enqueued with
    :py:meth:`process` method, no other job can be processed before its reply
    is received and picked up by the caller.

    :param reply_cond: Is a condition object used to notify any other thread
        about job's completion. This object may be used together with
        asynchronous call to :py:meth:`process` method to do some job in
        parallel and wait on thise condition afterwards until the job is
        answered by *YumWorker*.
    :type reply_cond: :py:class:`threading.Condition`
    """

    def __init__(self, reply_cond=None):
        threading.Thread.__init__(self, name="SessionManager")

        #: Recursive lock guarding concurently accessible attributes.
        self._busy_lock = threading.RLock()
        #: Condition used to notify main loop of this thread about new job
        #: available. It's also used in main loop to notify caller about
        #: reply being received and ready for picking up.
        self._busy_cond = threading.Condition(self._busy_lock)
        #: Currently processed job. ``None`` if no job is being processed.
        #: Pointer is reset to ``None`` when the reply is received.
        self._job = None

        #: Number of sessions currently active.
        self._session_level = 0

        #: Separated process accessing yum's api and handling jobs. Initialized
        #: when first needed. If this is ``False``, clean up request has been
        #: received. If ``None``, process has not been started yet or it was
        #: killed.
        self._process = None
        #: Number of resurrections for job currently processed.
        self._resurrections = 0
        #: This is filled with the answer from YumWorker. It's reset to ``None``
        #: when picked up by caller. It needs to be picked up before new job
        #: can be enqueued.
        self._reply = None

        if reply_cond is None:
            reply_cond = threading.Condition(threading.RLock())
        #: Used to wait for job to be processed and received.
        self._reply_cond = reply_cond

    @cmpi_logging.trace_method
    def _handle_reply_timeout(self, job):
        """
        This is called when timeout occurs while waiting on downlink queue for
        reply. Delay can be caused by worker process's early termination (bug).
        This handler tries to recover from such an situation.
        """
        if not self._worker.is_alive():
            if self._worker.exitcode < 0:
                msg = "worker process(pid=%d) killed by signal %s" % (
                    self._worker.pid, get_signal_name(-self._process.exitcode))
            else:
                msg = "worker process(pid=%d) is dead - exit code: %d" % (
                    self._process.pid, self._worker.exitcode)
            LOG().error("[jobid=%d] %s", job.jobid, msg)
            with self._busy_lock:
                self._resurrections += 1
                self._process = None
                LOG().error("[jobid=%d] starting new worker process", job.jobid)
                if not isinstance(job, jobs.YumLock):
                    if self._session_level > 0:
                        LOG().info('restoring session level=%d',
                                self._session_level)
                        new_session_job = jobs.YumLock()
                        self._worker.uplink.put(new_session_job)
                        reply = self._worker.downlink.get()
                        jobs.log_reply_error(new_session_job, reply)
                if self._resurrections > MAX_RESURRECTIONS:
                    LOG().warn("[jobid=%d] process has been resurrected maximum"
                            " number of times (%d times), cancelling job",
                            job.jobid, MAX_RESURRECTIONS)
                    raise errors.TransactionError(
                            "failed to complete job: %s" % (msg))
                else:
                    self._worker.uplink.put(job)
        else:
            LOG().info("[jobid=%d] process is running, waiting some more",
                    job.jobid)

    @cmpi_logging.trace_method
    def _receive_reply(self, job):
        """
        Block on downlink queue to receive expected replies from worker
        process.

        In case, that worker process terminated due to some error, restart it,
        restore session and resend the job request.
        """
        timeout = Configuration.get_instance().get_safe(
                'Jobs', 'WaitCompleteTimeout', float)
        while self._process is not False:   # process terminated
            LOG().debug("[jobid=%d] blocking on downlink queue", job.jobid)
            try:
                reply = self._worker.downlink.get(block=True, timeout=timeout)
                LOG().debug("[jobid=%d] received reply", job.jobid)
                if job.jobid != reply[0]:
                    raise errors.JobError('expected job with id=%d, got %s' % (
                        job.jobid, reply[0]))
                if self._reply:
                    raise errors.JobError('can not overwrite not yet picked'
                            ' up reply "%s" with "%s"' % (self._reply, reply))
                self._resurrections = 0
                return reply
            except TQueue.Empty:
                LOG().warn("[jobid=%d] wait for job reply timeout"
                        "(%d seconds) occured", job.jobid, timeout)
                self._handle_reply_timeout(job)
        raise errors.TerminatingError("can not complete job %s,"
                " terminating ..." % job)

    @cmpi_logging.trace_method
    def _send_and_receive(self, job):
        """
        Sends a request to server and blocks until job is processed by
        *YumWorker* and reply is received. It then notifies the called waiting
        either in the :py:meth:`process` method or just on
        :py:attr:`_reply_cond`.
        """
        with self._busy_lock:
            self._worker.uplink.put(job)
            reply = self._receive_reply(job)
            with self._reply_cond:
                self._reply = reply
                self._busy_cond.notifyAll()
                self._reply_cond.notifyAll()
                return reply

    @property
    def _worker(self):
        """
        *YumWorker* process accessor. It's created upon first need.

        :returns: Process object.
        :rtype: :py:class:`lmi.software.yumdb.process.YumWorker`
        """
        if self._process is False:
            raise errors.TerminatingError("provider is terminating")
        if self._process is None:
            LOG().trace_info("starting YumWorker")
            uplink = Queue()
            downlink = Queue()
            self._process = YumWorker(uplink, downlink)
            self._process.start()
            LOG().trace_info("YumWorker started with pid=%s", self._process.pid)
        return self._process

    def peak(self):
        """
        :returns: Received reply, if any. ``None`` is returned otherwise.
            This keeps reply with this object. Reply has always following
            format: ::

                (jobid, result_code, result_data)

        :rtype: tuple
        """
        with self._reply_cond:
            return self._reply

    @property
    def got_reply(self):
        """
        :returns: ``True`` if there is reply waiting to be picked up.
        :rtype: boolean
        """
        reply = self.peak()
        return reply is not None

    @property
    def is_busy(self):
        """
        :returns: Whether there is a job being processed.
        :type: boolean
        """
        if self._busy_lock.acquire(False):
            ret = self._job is not None
            self._busy_lock.release()
            return ret
        return False

    @cmpi_logging.trace_method
    def pop_reply(self, no_wait=False):
        """
        Wait for reply to be received, pick it up and return it.

        :param boolean no_wait: Whether to raise an exception if
            the reply is not yet available. If ``False``, wait for its
            arrival if not yet received.
        :returns: Received reply in form ``(jobid, result_code, result_data)``.
        :rtype: tuple
        """
        reply = None
        if self._busy_lock.acquire(not no_wait):
            try:
                with self._reply_cond:
                    reply = self._reply
                    self._reply = None
                if not no_wait:
                    while reply is None:
                        self._busy_cond.wait()
                        with self._reply_cond:
                            reply = self._reply
                            self._reply = None
            finally:
                self._busy_lock.release()

        if reply is None:   # this is possible only if no_wait=True
            raise TQueue.Empty("reply has not been received yet")

        return reply

    @cmpi_logging.trace_method
    def process(self, job, sync=True):
        """
        Enqueue job, and optionally wait to reply, pick it up and return it.

        :param boolean sync: Whether to wait for reply or leave immediately
            after the job is enqueued.
        :returns: Received reply if the *sync* is ``True``.
        :rtype: boolean
        """
        with self._busy_lock:
            # enqueueing job
            while self.is_busy or self._reply:
                self._busy_cond.wait()
            self._job = job
            self._busy_cond.notifyAll()
            if sync:
                # waiting for reply
                reply = self.pop_reply()
                assert reply[0] == job.jobid
                LOG().trace_info('gor reply from YumWorker: %s', reply)
                return reply

    @property
    def session_level(self):
        """
        Number of sessions currently active.
        """
        with self._busy_lock:
            return self._session_level

    @cmpi_logging.trace_method
    def begin_session(self):
        """
        Start a new session. This locks up yum database, if there has been
        not active session yet.
        """
        with self._busy_lock:
            if self._session_level == 0:
                self.process(jobs.YumLock())
                LOG().trace_info('new session started')
            self._session_level += 1
            LOG().trace_info('nested to session level=%d', self._session_level)

    @cmpi_logging.trace_method
    def end_session(self):
        """
        Stop session. This unlocks yum database, if there are any active
        sessions.
        """
        with self._busy_lock:
            if self._session_level == 1:
                self.process(jobs.YumUnlock())
                LOG().trace_info('session ended')
            LOG().trace_info('emerged from session level=%d',
                    self._session_level)
            self._session_level = max(self._session_level - 1, 0)

    def run(self):
        """
        Main loop of session manager. It resends all enqueued jobs to YumWorker,
        accepts replies and notifies callers.
        """
        LOG().info('%s started', self.name)
        while True:
            with self._busy_lock:
                while not self._job:
                    self._busy_cond.wait()
                job = self._job
                LOG().debug('sending job %s to YumWorker', job)
                if not isinstance(job, jobs.YumShutDown):
                    self._send_and_receive(job)
                    self._job = None
                else:   # last job - terminate Yum Worker
                    if self._process:
                        self._process.uplink.put(job)
                        self._process.join()
                        LOG().info('YumWorker terminated')
                    # prohibit next instantiation
                    self._process = False
                    with self._reply_cond:
                        self._reply = (
                                job.jobid, jobs.YumJob.RESULT_SUCCESS, None)
                        self._busy_cond.notifyAll()
                        self._reply_cond.notifyAll()
                    self._job = None
                    break
        LOG().info('%s terminated', self.name)



#!/usr/bin/python
# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details. #
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
"""
Abstraction of YUM repository for test purposes.
"""

from collections import defaultdict
from datetime import datetime
import inspect
import os
import re
from subprocess import call, check_output

import util

RE_REPO_TAG = re.compile(
    ur'^repo-(?P<tag>[a-z_-]+)[\s\u00a0]*:'
    ur'[\s\u00a0]*(?P<value>.*?)[\s\u00a0]*$', re.I | re.U | re.M)
RE_REPO_INFOS = re.compile(
    r'^(repo-id\s*:.*?)(?:^\s*$)',
    re.IGNORECASE | re.MULTILINE | re.DOTALL)
RE_REPO_CONFIG = re.compile(
    ur'^(?P<tag>[a-z_-]+)[ \t\u00a0]*='
    ur'[ \t\u00a0]*((?P<value>.*?)[ \t\u00a0]*)?$',
    re.I | re.M | re.U)
RE_REPO_ENABLED = re.compile(
    r'^enabled\s*=\s*(true|false|0|1|yes|no)\s*$', re.I | re.M)

class Repository(object):

    def __init__(self, repoid, name, status, revision,
            tags, last_updated, pkg_count, base_urls,
            metalink, mirror_list, filename, cost, timeout,
            gpg_check, repo_gpg_check, metadata_expire,
            config_path=None, packages=None):
        for attr in ('repoid', 'name', 'tags', 'metalink', 'filename'):
            setattr(self, attr, locals()[attr])
        for attr in ('gpg_check', 'repo_gpg_check', 'cost', 'status',
                'revision', 'last_updated', 'pkg_count',
                'base_urls', 'mirror_list', 'timeout', 'metadata_expire'):
            setattr(self, '_'+attr, None)
        self.base_urls = base_urls
        self.cost = cost
        self.gpg_check = gpg_check
        self.last_updated = last_updated
        self.metadata_expire = metadata_expire
        self.mirror_list = mirror_list
        self.pkg_count = pkg_count
        self.repo_gpg_check = repo_gpg_check
        self.revision = revision
        self.status = status
        self.timeout = timeout
        if config_path is None:
            repos_dir = os.environ.get('LMI_SOFTWARE_YUM_REPOS_DIR',
                    '/etc/yum.repos.d')
            config_path = os.path.join(repos_dir, '%s.repo' % self.repoid)
        self.config_path = config_path
        if packages is None:
            packages = set()
        self._packages = set(packages)

    @property
    def base_urls(self):
        return self._base_urls
    @base_urls.setter
    def base_urls(self, base_urls):
        if not isinstance(base_urls, list):
            raise TypeError('base_urls must be a list')
        self._base_urls = base_urls

    @property
    def local_path(self):
        if (   len(self._base_urls) == 1
           and self._base_urls[0].startswith('file://')):
            return self._base_urls[0][len('file://'):]

    @property
    def cost(self):
        return self._cost
    @cost.setter
    def cost(self, cost):
        self._cost = int(cost)

    @property
    def gpg_check(self):
        return self._gpg_check
    @gpg_check.setter
    def gpg_check(self, value):
        self._gpg_check = bool(value)

    @property
    def last_updated(self):
        return self._last_updated
    @last_updated.setter
    def last_updated(self, last_updated):
        if isinstance(last_updated, int):
            last_updated = datetime.fromtimestamp(last_updated)
        if last_updated is not None and not isinstance(last_updated, datetime):
            raise TypeError('last_updated must be a datetime object')
        self._last_updated = last_updated

    @property
    def metadata_expire(self):
        return self._metadata_expire
    @metadata_expire.setter
    def metadata_expire(self, metadata_expire):
        if not isinstance(metadata_expire, (int, long, basestring)):
            raise TypeError('metadata_expire must be either integer or datetime')
        self._metadata_expire = (
                     int(metadata_expire.strip().split(' ')[0])
                if   isinstance(metadata_expire, basestring)
                else metadata_expire)

    @property
    def mirror_list(self):
        return self._mirror_list
    @mirror_list.setter
    def mirror_list(self, mirror_list):
        if mirror_list is not None and not isinstance(mirror_list, basestring):
            raise TypeError('mirror list must be a string')
        if not mirror_list:
            mirror_list = None
        self._mirror_list = mirror_list

    @property
    def packages(self):
        return self._packages

    @property
    def pkg_count(self):
        return self._pkg_count
    @pkg_count.setter
    def pkg_count(self, pkg_count):
        self._pkg_count = None if pkg_count is None else int(pkg_count)

    @property
    def repo_gpg_check(self):
        return self._repo_gpg_check
    @repo_gpg_check.setter
    def repo_gpg_check(self, value):
        self._repo_gpg_check = bool(value)

    @property
    def revision(self):
        return self._revision
    @revision.setter
    def revision(self, value):
        self._revision = None if value is None else int(value)

    @property
    def status(self):
        return self._status
    @status.setter
    def status(self, value):
        self._status = bool(value)

    @property
    def timeout(self):
        return self._timeout
    @timeout.setter
    def timeout(self, timeout):
        self._timeout = float(timeout)

    def __getitem__(self, pkg_name):
        pkg_dict = {p.name : p for p in self._packages}
        try:
            return pkg_dict[pkg_name]
        except KeyError:
            try:
                return pkg_dict['openlmi-sw-test-' + pkg_name]
            except KeyError:
                raise KeyError(pkg_name)

    def refresh(self):
        updated_attrs = set()
        for attr, value in _parse_repo_file(self.repoid).items():
            try:
                setattr(self, attr, value)
                updated_attrs.add(attr)
            except AttributeError:
                print 'can\'t set attribute "%s"' % attr
        if not self.status:
            for attr in ('revision', 'tags', 'last_updated'):
                if attr not in updated_attrs and getattr(self, attr) is not None:
                    setattr(self, attr, None)

def to_json(encoder, repo):
    """
    Converts repository object to dictionary which json encoder can handle.

    :param encoder: Instance of json encoder.
    :param repo: Package object to convert.
    :type repo: :py:class:`Repository`
    :returns: Dictionary with repository attributes.
    :rtype: dictionary
    """
    if not isinstance(repo, Repository):
        raise TypeError("not a repo object")
    def transform_value(val):
        if not isinstance(val, (int, long, float, tuple, list,
                dict, bool, basestring)) and val is not None:
            return encoder.default(val)
        if isinstance(val, (list, tuple)):
            return type(val)((transform_value(v) for v in val))
        return val
    argspec = inspect.getargspec(repo.__init__)
    return { a: transform_value(getattr(repo, a)) for a in argspec.args[1:] }

def from_json(package_deserializer, json_object):
    """
    Constructs a repository object from dictionary loaded from json text.
    Inverse function to :py:func:`to_json`.

    :param callable package_deserializer: Function taking encoder and package
        dictionary as arguments, returning new package object.
    :param dictionary json_object: Deserialized repository as a dictionary.
    :returns: Package object.
    :rtype: :py:class:`Package`
    """
    if 'gpg_check' in json_object:
        kwargs = {}
        for key, value in json_object.items():
            if isinstance(value, (tuple, list)):
                value = type(value)((package_deserializer(v) for v in value))
            else:
                value = package_deserializer(value)
            kwargs[key] = value
        return Repository(**kwargs)
    return json_object

# example of repo information
#Repo-id      : updates-testing/18/x86_64
#Repo-name    : Fedora 18 - x86_64 - Test Updates
#Repo-status  : enabled
#Repo-revision: 1360887143
#Repo-tags    : binary-x86_64
#Repo-updated : Fri Feb 15 02:02:20 2013
#Repo-pkgs    : 13 808
#Repo-size    : 14 G
#Repo-baseurl : file:///mnt/globalsync/fedora/linux/updates/testing/18/x86_64
#Repo-expire  : 21 600 second(s) (last: Fri Feb 15 10:32:13 2013)
#Repo-filename: ///etc/yum.repos.d/fedora-updates-testing.repo

def _parse_repo_file(repo_name):
    """
    Parse output of ``yum-config-manager`` command for single repository.

    :param string repo_name: Repository id.
    :returns: Dictionary with key-value pairs suitable for passing to
        :py:class:`Repository` constructor.
    :rtype: dictionary
    """
    cmd = ["yum-config-manager", repo_name]
    out = check_output(cmd).decode('utf-8')
    result = {}
    for match in RE_REPO_CONFIG.finditer(out):
        tag = match.group('tag')
        value = match.group('value')
        if tag == "gpgcheck":
            result["gpg_check"] = value.lower() == "true"
        elif tag == "repo_gpgcheck":
            result["repo_gpg_check"] = value.lower() == "true"
        elif tag == "timeout":
            result[tag] = float(value)
        elif tag == "metadata_expire":
            result[tag] = int(value)
        elif tag == "cost":
            result[tag] = int(value)
        elif tag == 'baseurl':
            result['base_urls'] = [value]
        elif tag == 'enabled':
            result['status'] = value.lower() in ('true', 'yes', 1)
        elif tag == 'mirrorlist':
            result['mirror_list'] = value
        elif tag in {p for p in Repository.__dict__ if not p.startswith('_')}:
            continue
    return result

def make_repo(repo_info, packages=None):
    """
    Makes a Repository instance from string dumped by yum repoinfo command.
    """
    metadata = defaultdict(lambda : None)
    metadata["base_urls"] = []
    argspec = inspect.getargspec(Repository.__init__)
    for match in RE_REPO_TAG.finditer(repo_info):
        tag = match.group('tag')
        value = match.group('value')
        try:
            new_tag = \
                { 'id'      : 'repoid'
                , 'updated' : 'last_updated'
                , 'pkgs'    : 'pkg_count'
                , 'baseurl' : 'base_urls'
                , 'mirrors' : 'mirror_list'
                }[tag]
        except KeyError:
            if tag not in argspec.args:
                continue # unexpeted, or unwanted tag found
            new_tag = tag
        if new_tag == 'repoid':
            metadata[new_tag] = value.split('/')[0]
        elif new_tag == "base_urls":
            metadata[new_tag].append(value)
        elif new_tag == "last_updated":
            metadata[new_tag] = datetime.strptime(
                    value, "%a %b %d %H:%M:%S %Y")
        elif new_tag == "pkg_count":
            metadata[new_tag] = int(
                    u"".join(re.split(ur'[\s\u00a0]+', value, re.UNICODE)))
        elif new_tag == "status":
            metadata[new_tag] = value == "enabled"
        elif new_tag == 'revision':
            metadata[new_tag] = int(value)
        else:
            metadata[new_tag] = value
    if packages is None:
        packages = set()
    metadata['packages'] = packages
    config_items = _parse_repo_file(metadata['repoid'])
    for field in argspec.args[1:]:  # leave out self argument
        if not field in metadata:
            metadata[field] = config_items.get(field, None)
    return Repository(**metadata)

def get_repo_list(kind='all'):
    if kind.lower() not in ('all', 'enabled', 'disabled'):
        raise ValueError('kind must be on of {"all", "enabled", "disabled"}')
    cmd = ["yum", "-q", "repoinfo", "all"]
    env = os.environ.copy()
    env['LC_ALL'] = 'C'
    return list(
                RE_REPO_TAG.search(m.group(1)).group('value').split('/')[0]
            for m in RE_REPO_INFOS.finditer(
                        check_output(cmd, env=env).decode('utf-8')))

def get_repo_database():
    """
    :returns: List of repository objects for all configured repositories.
    :rtype: list
    """
    result = []
    cmd = ["yum", "-q", "repoinfo", "all"]
    env = os.environ.copy()
    env['LC_ALL'] = 'C'
    repo_infos = check_output(cmd, env=env).decode('utf-8')
    for match in RE_REPO_INFOS.finditer(repo_infos):
        result.append(make_repo(match.group(1)))
    return result

def is_repo_enabled(repo):
    """
    :param repo: Either a repository id or instance of :py:class:`Repository`.
    :returns: Whether the repository is enabled.
    :rtype: boolean
    """
    if isinstance(repo, Repository):
        repo = repo.repoid
    cmd = ["yum-config-manager", repo]
    out = check_output(cmd)
    match = RE_REPO_ENABLED.search(out)
    return bool(match) and match.group(1).lower() in {"true", "yes", "1"}

def set_repos_enabled(repos, enable=True):
    """
    Enables or disables repository in its configuration file.

    :param repo: Eiether a repository id or instance of :py:class:`Repository`.
    :param boolean enable: New state of repository.
    """
    cmd = ["yum-config-manager", "--enable" if enable else "--disable"]
    repoids = []
    if isinstance(repos, (basestring, Repository)):
        repos = [repos]
    for repo in repos:
        if isinstance(repo, Repository):
            repo = repo.repoid
        repoids.append(repo)
    if len(repoids) > 0:
        cmd.extend(repoids)
        call(cmd, stdout=util.DEV_NULL, stderr=util.DEV_NULL)




# -*- encoding: utf-8 -*-
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Roman Rakus <rrakus@redhat.com>
#

from common import AccountBase
from methods import *
import subprocess

class TestAccount(AccountBase):
    """
    Class for testing LMI_Account class
    """
    def test_account_properties(self):
        """
        Account: Test if there are key and main properties in LMI_Account
        """
        slct = "select * from LMI_Account"
        instances = self.wbemconnection.ExecQuery('WQL', slct)
        self.assertTrue(len(instances) > 0)
        # check if it provides key properties
        for attr in ["CreationClassName", "Name", "SystemCreationClassName",
                     "SystemName"]:
            self.assertIsNotNone(instances[0][attr])

        # check if it provides other properties, which should be set
        for attr in ["host", "UserID", "UserPassword", "UserPasswordEncoding"]:
            self.assertIsNotNone(instances[0][attr])

    def test_create_account(self):
        """
        Account: Test to create user account
        """
        # make sure the account will not exist
        clean_account(self.user_name)
        computer_system = self.wbemconnection.ExecQuery('WQL',
            'select * from %s' % self.system_cs_name)[0]
        lams = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_AccountManagementService')[0]
        self.wbemconnection.InvokeMethod("CreateAccount", lams.path,
            Name=self.user_name, System=computer_system.path)
        # The user now should be created, check it
        self.assertTrue(user_exists(self.user_name))
        # now delete that user
        clean_account(self.user_name)

    def test_delete_account(self):
        """
        Account: Test to delete account
        """
        # make sure the account will exist
        create_account(self.user_name)
        i = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_Account where Name = "%s"' % self.user_name)[0]
        self.wbemconnection.DeleteInstance(i.path)
        # check if it was really deleted
        self.assertFalse(user_exists(self.user_name))
        clean_account(self.user_name)

    def test_modify_account(self):
        """
        Account: Test several modifications
        """
        create_account(self.user_name)
        i = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_Account where Name = "%s"' % self.user_name)[0]
        # gecos
        i["ElementName"] = "GECOS"
        self.wbemconnection.ModifyInstance(i)
        self.assertEqual(field_in_passwd(self.user_name, 4), "GECOS")
        # login shell
        i["LoginShell"] = "/modified"
        self.wbemconnection.ModifyInstance(i)
        self.assertEqual(field_in_passwd(self.user_name, 6), "/modified")
        # password
        i["UserPassword"][0] = '$6$9Ky8vI6f$ipRcdc7rgMrtDh.sWOaRSoBck2cLz4eUom8Eze.NaY2DoMmNimuFBrXpJjlPCjMoeFTYC.FdZwj488JZcohyw1'
        self.wbemconnection.ModifyInstance(i)
        self.assertEqual(field_in_shadow(self.user_name, 1),
            '$6$9Ky8vI6f$ipRcdc7rgMrtDh.sWOaRSoBck2cLz4eUom8Eze.NaY2DoMmNimuFBrXpJjlPCjMoeFTYC.FdZwj488JZcohyw1')
        clean_account(self.user_name)




# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#
"""
Module holding an abstraction for RPM package.
"""

from datetime import datetime
import yum

from lmi.software import util
from lmi.software.yumdb.util import is_pkg_installed

class PackageInfo(object):
    """
    Container for package metadata. It represents rpm package in yum
    database. It's supposed to be passed from YumWorker to YumDB client
    and vice-versa. Instances of YumAvailablePackage can not be exchanged
    -- results in segfaults.

    To speed up looking up of original yum package object on server, an
    atribute "objid" is provided.
    """
    __slots__ = (
            "objid",
            "name", "epoch", "version", "release", "architecture",
            'summary', 'description', 'license', 'group', 'vendor',
            "repoid", 'size',
            'installed',    # boolean
            'install_time'  # datetime instance
    )

    def __init__(self, objid, name, epoch, version, release, arch, **kwargs):
        """
        @param objid is an in of original yum package object, which is used
        by server for subsequent operations on this package requested by client
        """
        self.objid = objid
        self.name = name
        self.epoch = epoch
        self.version = version
        self.release = release
        self.architecture = arch
        self.summary = kwargs.pop('summary', None)
        self.description = kwargs.pop('description', None)
        self.license = kwargs.pop('license', None)
        self.group = kwargs.pop('group', None)
        self.vendor = kwargs.pop('vendor', None)
        self.repoid = kwargs.pop("repoid", None)
        self.size = kwargs.pop('size', None)
        if self.size is not None and not isinstance(self.size, (int, long)):
            raise TypeError('size must be an integer')
        self.installed = kwargs.pop('installed', None)
        if self.installed is not None:
            self.installed = bool(self.installed)
        self.install_time = kwargs.pop('install_time', None)
        if (   self.install_time is not None
           and not isinstance(self.install_time, datetime)):
            raise TypeError('install_time must be a datetime')

    # *************************************************************************
    # Properties
    # *************************************************************************
    @property
    def ver(self):
        """Shortcut for version property."""
        return self.version

    @property
    def rel(self):
        """Shortcut for release property."""
        return self.release

    @property
    def arch(self):
        """Shortcut for architecture property."""
        return self.architecture

    @property
    def nevra(self):
        """@return nevra of package with epoch always present."""
        return self.get_nevra(with_epoch="ALWAYS")

    @property
    def evra(self):
        """@return evra of package."""
        return "%s:%s-%s.%s" % (
                self.epoch if self.epoch and self.epoch != "(none)" else "0",
                self.version,
                self.release,
                self.architecture)

    @property
    def key_props(self):
        """
        @return package properties as dictionary,
        that uniquelly identify package in database
        """
        return dict((k, getattr(self, k)) for k in (
            'name', 'epoch', 'version', 'release', 'arch', 'repoid'))

    # *************************************************************************
    # Public methods
    # *************************************************************************
    def get_nevra(self, with_epoch='NOT_ZERO'):
        """
        @param with_epoch may be one of:
            "NOT_ZERO" - include epoch only if it's not zero
            "ALWAYS"   - include epoch always
            "NEVER"    - do not include epoch at all
        @return nevra of package
        """
        return util.make_nevra(self.name, self.epoch, self.version,
                self.release, self.arch, with_epoch)

    # *************************************************************************
    # Special methods
    # *************************************************************************
    def __str__(self):
        return self.nevra

    def __getstate__(self):
        """
        Used for serialization with pickle.
        @return container content that will be serialized
        """
        return dict((k, getattr(self, k)) for k in self.__slots__)

    def __setstate__(self, state):
        """
        Used for deserialization with pickle.
        Restores the object from serialized form.
        @param state is an object created by __setstate__() method
        """
        for k, value in state.items():
            setattr(self, k, value)

    def __eq__(self, other):
        return (   self.name == other.name
               and self.version == other.version
               and self.release == other.release
               and self.arch == other.arch
               and self.epoch == other.epoch
               and (  (self.repoid is None or other.repoid is None)
                   or (self.repoid == other.repoid)))

def make_package_from_db(pkg, rpmdb):
    """
    Create instance of :py:class:`PackageInfo` from instance of
    :py:class:`yum.packages.PackageObject`.

    :param pkg: Yum package object.
    :type pkg: :py:class:`yum.packages.PackageObject`
    :param rpmdb: Installed package sack.
    :type rpmdb: :py:class:`yum.rpmsack.RPMDBPackageSack`
    :rtype: :py:class:`PackageInfo`
    """
    metadata = dict((k, getattr(pkg, k)) for k in (
        'summary', 'description', 'license', 'group', 'vendor', 'size',
        'repoid'))
    if is_pkg_installed(pkg, rpmdb):
        metadata['installed'] = True
        if isinstance(pkg, yum.sqlitesack.YumAvailablePackageSqlite):
            metadata['install_time'] = datetime.fromtimestamp(
                    rpmdb._tup2pkg[pkg.pkgtup].installtime)
        else:
            metadata['install_time'] = datetime.fromtimestamp(pkg.installtime)
    else:
        metadata['installed'] = False
    res = PackageInfo(id(pkg), pkg.name, pkg.epoch, pkg.version, pkg.release,
            pkg.arch, **metadata)
    return res




# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""
Common utilities concerning SystemSoftwareCollection provider.
"""

import pywbem

from lmi.providers import cmpi_logging
from lmi.software import util

def get_path():
    """@return instance name with prefilled properties"""
    op = util.new_instance_name("LMI_SystemSoftwareCollection",
            InstanceID="LMI:LMI_SystemSoftwareCollection")
    return op

@cmpi_logging.trace_function
def check_path(env, collection, prop_name):
    """
    Checks instance name of SystemSoftwareCollection.
    @param prop_name name of object name; used for error descriptions
    """
    if not isinstance(collection, pywbem.CIMInstanceName):
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "\"%s\" must be a CIMInstanceName" % prop_name)
    our_collection = get_path()
    ch = env.get_cimom_handle()
    if collection.namespace != our_collection.namespace:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                'Namespace of "%s" does not match "%s"' % (
                    prop_name, our_collection.namespace))
    if not ch.is_subclass(our_collection.namespace,
            sub=collection.classname,
            super=our_collection.classname):
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "Class of \"%s\" must be a sublass of %s" % (
                    prop_name, our_collection.classname))
    if not 'InstanceID' in collection:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "\"%s\" is missing InstanceID key property", prop_name)
    if collection['InstanceID'] != our_collection['InstanceID']:
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                "InstanceID of \"%s\" does not match \"%s\"" %
                prop_name, our_collection['InstanceID'])
    return True

@cmpi_logging.trace_function
def check_path_property(env, op, prop_name):
    """
    Checks, whether prop_name property of op object path is correct.
    If not, an exception will be risen.
    """
    if not prop_name in op:
        raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                "Missing %s key property!" % prop_name)
    return check_path(env, op[prop_name], prop_name)



# -*- Coding:utf-8 -*-
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
"""
Contains handlers and loggers specific to providers logging running
under cimom.
"""

import logging
import functools
import inspect
from itertools import chain
import os
import sys

# Custom logging levels
TRACE_WARNING = logging.DEBUG - 1
TRACE_INFO = logging.DEBUG - 2
TRACE_VERBOSE = logging.DEBUG - 3

#: Mapping from level name to its number.
LOGGING_LEVELS = {
        "critical"      : logging.CRITICAL,
        "error"         : logging.ERROR,
        "warning"       : logging.WARNING,
        "warn"          : logging.WARNING,
        "info"          : logging.INFO,
        "debug"         : logging.DEBUG,
        "trace_warning" : TRACE_WARNING,
        "trace_info"    : TRACE_INFO,
        "trace_verbose" : TRACE_VERBOSE
}

#: This associates special format strings to various logger names
SPECIAL_FORMAT_STRINGS = {
    "lmi.providers.cmpi_logging.trace_function_or_method" :
            "%(levelname)s:%(message)s"
}

#: Default format string to use in stderr and cmpi handlers.
DEFAULT_FORMAT_STRING = \
    "%(levelname)s:%(module)s:%(funcName)s:%(lineno)d - %(message)s"

class DispatchingFormatter(object):
    """
    Formatter class for logging module. It allows to predefine different
    format string for paricular module names.

    There is no way, how to setup this formatter in configuration file.
    """
    def __init__(self, formatters, default):
        """
        *format* in parameters description can be either ``string`` or
        another formatter object.

        :param formatters (``dict``) Mapping of module names to *format*.
        :param default Default *format*.
        """
        for k, formatter in formatters.items():
            if isinstance(formatter, basestring):
                formatters[k] = logging.Formatter(formatter)
        self._formatters = formatters
        if isinstance(default, basestring):
            default = logging.Formatter(default)
        self._default_formatter = default

    def format(self, record):
        """
        Interface for logging module.
        """
        formatter = self._formatters.get(record.name, self._default_formatter)
        return formatter.format(record)

class CMPILogHandler(logging.Handler):
    """
        A handler class, which sends log messages to CMPI log.
    """

    def __init__(self, cmpi_logger, *args, **kwargs):
        self.cmpi_logger = cmpi_logger
        super(CMPILogHandler, self).__init__(*args, **kwargs)

    def emit(self, record):
        msg = self.format(record)
        if record.levelno >= logging.ERROR:
            self.cmpi_logger.log_error(msg)
        elif record.levelno >= logging.WARNING:
            self.cmpi_logger.log_warn(msg)
        elif record.levelno >= logging.INFO:
            self.cmpi_logger.log_info(msg)
        elif record.levelno >= logging.DEBUG:
            self.cmpi_logger.log_debug(msg)
        elif record.levelno >= TRACE_WARNING:
            self.cmpi_logger.trace_warn(record.filename, msg)
        elif record.levelno >= TRACE_INFO:
            self.cmpi_logger.trace_info(record.filename, msg)
        elif record.levelno >= TRACE_VERBOSE:
            self.cmpi_logger.trace_verbose(record.filename, msg)

class CMPILogger(logging.getLoggerClass()):
    """
        A logger class, which adds trace_method level log methods.
    """
    def trace_warn(self, msg, *args, **kwargs):
        """ Log message with TRACE_WARNING severity. """
        self.log(TRACE_WARNING, msg, *args, **kwargs)

    def trace_info(self, msg, *args, **kwargs):
        """ Log message with TRACE_INFO severity. """
        self.log(TRACE_INFO, msg, *args, **kwargs)

    def trace_verbose(self, msg, *args, **kwargs):
        """ Log message with TRACE_VERBOSE severity. """
        self.log(TRACE_VERBOSE, msg, *args, **kwargs)

    def _log(self, level, msg, args, exc_info=None, extra=None):
        """
        Overrides ``_log()`` function of basic ``Logger``. The purpose is to
        log tracebacks with different level instead of ERROR to prevent them
        being logged to syslog.
        """
        if logging._srcfile:
            #IronPython doesn't track Python frames, so findCaller throws an
            #exception on some versions of IronPython. We trap it here so that
            #IronPython can use logging.
            try:
                fn, lno, func = self.findCaller()
            except ValueError:
                fn, lno, func = "(unknown file)", 0, "(unknown function)"
        else:
            fn, lno, func = "(unknown file)", 0, "(unknown function)"
        if exc_info:
            if not isinstance(exc_info, tuple):
                exc_info = sys.exc_info()
            record = self.makeRecord(self.name, level, fn, lno, msg, args,
                    None, func, extra)
            self.handle(record)
            record = self.makeRecord(self.name, TRACE_WARNING, fn,
                    lno, str(exc_info[1]), tuple(), exc_info, func, extra)
        else:
            record = self.makeRecord(self.name, level, fn, lno, msg,
                    args, exc_info, func, extra)
        self.handle(record)

logging.setLoggerClass(CMPILogger)

def render_value(val):
    """
    When logging values, we want to avoid excessively long messages caused
    by rendering argument values like lists, dictionaries etc.
    Let's shorten these iterable objects to just one or few items.

    :param val: Any value for rendering.
    :returns: Representation string of value, possibly shortened.
    :rtype: string
    """
    if isinstance(val, list):
        if len(val) < 2:
            return repr(val)
        else:
            return "[%s, ... (%d more items)]" % (
                    render_value(val[0]), len(val) - 1)
    elif isinstance(val, dict):
        if len(val) < 2:
            return repr(val)
        else:
            key = next(iter(val))
            return '{%s: %s, ... (%d more items)}' % (
                    render_value(key), render_value(val[key]), len(val) - 1)
    elif isinstance(val, set):
        if len(val) < 2:
            return repr(val)
        else:
            return '{%s, ... (%d more items)}' % (
                    render_value(val[0]), len(val) - 1)
    elif isinstance(val, tuple):
        return "(%s%s)" % (
                ", ".join(render_value(i) for i in val),
                ", " if len(val) < 2 else '')
    return repr(val)

def _trace_function_or_method(is_method=False, frame_level=1):
    """
    Factory for function and method decorators. Generated decorators
    log every calls and exits of decorated functions or methods.

    Logged information contain the caller's module and line together with
    called function's module, function name and line number.

    :param boolean is_method: Whether returned decorator is targeted
        for use upon a method of a class. It modified logged function by
        prepending owning class name.
    :param integer frame_level: Number of nested frames to skip when
        searching for called function scope by inspecting stack upwards.
        When the result of this function is applied directly on the definition
        of function, it's value should be 1. When used from inside of some
        other factory, it must be increased by 1.
    """

    assert frame_level >= 1

    def _decorator(func):
        """
        Decorator for logging entries and exits of function or method.
        """
        if not inspect.ismethod(func) and not inspect.isfunction(func):
            raise TypeError("func must be a function")

        module = func.__module__.split('.')[-1]
        frm = inspect.currentframe()
        for _ in range(frame_level):
            frm = frm.f_back
        lineno = frm.f_lineno
        del frm
        classname = inspect.getouterframes(
                inspect.currentframe())[frame_level][3]

        @functools.wraps(func)
        def _wrapper(*args, **kwargs):
            """
            Wrapper for function or method, that does the logging.
            """
            logger = logging.getLogger(__name__+'.trace_function_or_method')
            logargs = {}
            if logger.isEnabledFor(TRACE_VERBOSE):
                frm = inspect.currentframe()
                logargs.update({
                    "caller_file" : os.path.basename(os.path.splitext(
                        frm.f_back.f_code.co_filename)[0]),
                    "caller_lineno" : frm.f_back.f_lineno,
                    "module" : module,
                    "func"   : classname + "." + func.__name__
                        if is_method else func.__name__,
                    "lineno" : lineno,
                    "action" : "entering",
                    "args"   : ", ".join(chain(
                                (render_value(a) for a in args),
                                (   "%s=%s"%(k, render_value(v))
                                for k, v in kwargs.items())))
                })

                if not logargs["args"]:
                    logargs["args"] = ""
                else:
                    logargs["args"] = " with args=(%s)" % logargs["args"]
                logger.trace_verbose("%(caller_file)s:%(caller_lineno)d - "
                    "%(action)s %(module)s:%(func)s:%(lineno)d%(args)s",
                    logargs)
            try:
                result = func(*args, **kwargs)
                if logger.isEnabledFor(TRACE_VERBOSE):
                    logargs["action"] = "exiting"
                    logger.trace_verbose("%(caller_file)s:%(caller_lineno)d"
                        " - %(action)s %(module)s:%(func)s:%(lineno)d",
                        logargs)
            except Exception as exc:
                if logger.isEnabledFor(TRACE_VERBOSE):
                    logargs['action'] = 'exiting'
                    logargs['error'] = str(exc)
                    logger.trace_verbose("%(caller_file)s:%(caller_lineno)d"
                        " - %(action)s %(module)s:%(func)s:%(lineno)d"
                        " with error: %(error)s", logargs)
                raise
            return result

        return _wrapper

    return _decorator

def _trace_function(func, frame_level=1):
    """ Convenience function used for decorating simple functions. """
    return trace_function_or_method(frame_level=frame_level + 1)(func)

def _trace_method(func, frame_level=1):
    """ Convenience function used for decorating methods. """
    return trace_function_or_method(True, frame_level + 1)(func)

def _identity_decorator(func, *args, **kwargs):
    """ Decorator returning the function itself. """
    return func

# Tracing decorators may be disabled by environment variable.
if os.getenv("LMI_DISABLE_TRACING_DECORATORS", "0").lower() in \
        ("1", "true", "yes", "on"):
    # Tracing decators disabled. Functions and method won't get modified
    # in any way.
    trace_function_or_method = _identity_decorator
    trace_function           = _identity_decorator
    trace_method             = _identity_decorator
else:
    # Tracing decorators enabled.
    trace_function_or_method = _trace_function_or_method
    trace_function           = _trace_function
    trace_method             = _trace_method

def try_setup_from_file(config):
    """
    Try to configure logging with a file specified in configuration.

    :returns: ``True`` if the file configuration is given,  successfuly
        parsed and carried out.
    :rtype: boolean
    """
    try:
        path = config.file_path('Log', 'FileConfig')
        if not os.path.exists(path):
            logging.getLogger(__name__).error('given FileConfig "%s" does'
                    ' not exist', path)
        else:
            logging.config.fileConfig(path)
            return True
    except Exception:
        if config.config.has_option('Log', 'FileConfig'):
            logging.getLogger(__name__).exception(
                    'failed to setup logging from FileConfig')
    return False

def setup(env, config, special_format_strings=None):
    """
    Set up the logging with options stored in
    :py:class:`lmi.base.BaseConfiguration` instance. This should be called
    at provider's startup before any message is sent to log.

    :param ProviderEnvironment env: Provider environment, taken from CIMOM
        callback (e.g. ``get_providers()``).
    :param config:  Configuration with Log section containing settings for
        logging.
    :type config: :py:class:`lmi.base.BaseConfiguration`
    :param dictionary special_format_strings: Assignes to various loggers
        special format strings. It overrides pairs in
        :py:data:`SPECIAL_FORMAT_STRINGS`. Its format is following: ::

            { ( 'logger_name' : 'format_string_to_use' ), ... }
    """
    if (   special_format_strings is not None
       and not isinstance(special_format_strings, dict)):
        raise TypeError("special_format_strings must be a dictionary")
    if try_setup_from_file(config):
        return
    logging_level = logging.ERROR
    if not config.logging_level in LOGGING_LEVELS:
        logging.getLogger(__name__).error(
                'level name "%s" not supported', config.logging_level)
    else:
        logging_level = LOGGING_LEVELS[config.logging_level]
    logger = logging.getLogger()
    logger.setLevel(logging_level)
    # remove any previously set handlers
    for handler in logger.handlers[:]:
        logger.removeHandler(handler)
    format_strings = SPECIAL_FORMAT_STRINGS.copy()
    if special_format_strings is not None:
        format_strings.update(special_format_strings)
    # set up new ones
    if config.stderr:
        err_handler = logging.StreamHandler()
        err_handler.setLevel(logging_level)
        err_handler.setFormatter(
                DispatchingFormatter(format_strings, DEFAULT_FORMAT_STRING))
        logger.addHandler(err_handler)
    cmpi_handler = CMPILogHandler(env.get_logger(), logging_level)
    cmpi_handler.setFormatter(
            DispatchingFormatter(format_strings, DEFAULT_FORMAT_STRING))
    logger.addHandler(cmpi_handler)

class LogManager(object):
    """
        Class, which takes care of CMPI logging.
        There should be only one instance of this class and it should be
        instantiated as soon as possible, even before reading a config.
        The config file can be provided later by set_config call.

        Use of this manager is an alternative to single call to ``setup()``
        function of this module.
    """

    def __init__(self, env):
        """
            Initialize logging.
        """
        self._env = env
        self._config = None

    @property
    def config(self):
        """ Provider configuration object. """
        return self._config
    @config.setter
    def config(self, config):
        """
            Set a configuration of logging. It applies its setting immediately
            and also subscribes for configuration changes.
        """
        self._config = config
        config.add_listener(self._config_changed)
        # apply the config
        self._config_changed(config)

    @property
    def cmpi_handler(self):
        """ Returns cmpi log handler passing logged messages to cimom. """
        for handler in logging.getLogger('').handlers:
            if isinstance(handler, CMPILogHandler):
                return handler
        return CMPILogHandler(self._env.get_logger())

    @trace_method
    def _config_changed(self, config):
        """
            Apply changed configuration, i.e. start/stop sending to stderr
            and set appropriate log level.
        """
        setup(self._env, config)

    def destroy(self):
        """
            Shuts down the logging framework. No logging can be done
            afterwards.
        """
        logging.getLogger(__name__).debug('shutting down logging')
        logging.shutdown()

def get_logger(module_name):
    """
    Convenience function for getting callable returning logger for particular
    module name. It's supposed to be used at module's level to assign its
    result to global variable like this:

        LOG = cmpi_logging.get_logger(__name__)

    This can be used in module's functions and classes like this:

        def module_function(param):
            LOG().debug("this is debug statement logging param: %s", param)

    Thanks to ``LOG`` being a callable, it always returns valid logger object
    with current configuration, which may change overtime.
    """
    def _logger():
        """ Callable used to obtain current logger object. """
        return logging.getLogger(module_name)
    return _logger



from setuptools import setup
from lmi.base import __version__

setup(
    name='openlmi',
    description='OpenLMI python providers',
    author='Michal Minar',
    author_email='miminar@redhat.com',
    url='https://fedorahosted.org/openlmi/',
    version=__version__,
    license='LGPLv2+',
    namespace_packages = ['lmi'],
    packages = ['lmi', 'lmi.base', 'lmi.providers', 'lmi.test'],
    install_requires=['pywbem'],
    classifiers=[
        'License :: OSI Approved :: GNU Lesser General Public License'
            ' v2 or later (LGPLv2+)',
        'Operating System :: POSIX :: Linux',
        'Topic :: System :: Systems Administration',
        ]
    )



# -*- encoding: utf-8 -*-
# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Michal Minar <miminar@redhat.com>
#

"""
Entry module for OpenLMI Software providers.
"""

import logging
from multiprocessing import Queue

from lmi.base import singletonmixin
from lmi.providers import cmpi_logging
from lmi.providers import JobManager
from lmi.providers.IndicationManager import IndicationManager
from lmi.software import util
from lmi.software.core import Job
from lmi.software.LMI_SoftwareIdentity import LMI_SoftwareIdentity
from lmi.software.LMI_SystemSoftwareCollection import \
        LMI_SystemSoftwareCollection
from lmi.software.LMI_HostedSoftwareCollection import \
        LMI_HostedSoftwareCollection
from lmi.software.LMI_MemberOfSoftwareCollection import \
        LMI_MemberOfSoftwareCollection
from lmi.software.LMI_InstalledSoftwareIdentity import \
        LMI_InstalledSoftwareIdentity
from lmi.software.LMI_SoftwareIdentityResource import \
        LMI_SoftwareIdentityResource
from lmi.software.LMI_ResourceForSoftwareIdentity import \
        LMI_ResourceForSoftwareIdentity
from lmi.software.LMI_HostedSoftwareIdentityResource import \
        LMI_HostedSoftwareIdentityResource
from lmi.software.LMI_SoftwareInstallationService import \
        LMI_SoftwareInstallationService
from lmi.software.LMI_SoftwareInstallationServiceCapabilities import \
        LMI_SoftwareInstallationServiceCapabilities
from lmi.software.LMI_AssociatedSoftwareInstallationServiceCapabilities \
        import LMI_AssociatedSoftwareInstallationServiceCapabilities
from lmi.software.LMI_HostedSoftwareInstallationService import \
        LMI_HostedSoftwareInstallationService
from lmi.software.LMI_SoftwareInstallationServiceAffectsElement import \
        LMI_SoftwareInstallationServiceAffectsElement
from lmi.software.LMI_SoftwareJob import LMI_SoftwareJob
from lmi.software.LMI_SoftwareMethodResult import LMI_SoftwareMethodResult
from lmi.software.LMI_AffectedSoftwareJobElement import \
        LMI_AffectedSoftwareJobElement
from lmi.software.LMI_AssociatedSoftwareJobMethodResult import \
        LMI_AssociatedSoftwareJobMethodResult
from lmi.software.LMI_OwningSoftwareJobElement import \
        LMI_OwningSoftwareJobElement
from lmi.software.LMI_SoftwareIdentityFileCheck import \
        LMI_SoftwareIdentityFileCheck
from lmi.software.LMI_SoftwareIdentityChecks import LMI_SoftwareIdentityChecks
from lmi.software.yumdb import jobmanager
from lmi.software.yumdb import YumDB

def get_providers(env):
    """
    *CIMOM* callback.

    :returns: Mapping of provider names to corresponding provider instances.
    :rtype: dictionary
    """
    # first let's setup logging
    cmpi_logging.setup(env, util.Configuration.get_instance())

    # jobmanager does not understand CIM models, give it a way to transform
    # job to CIMIndication instance
    jobmanager.JOB_TO_MODEL = lambda job: Job.job2model(job, keys_only=False)

    providers = {
        "LMI_SoftwareIdentity"            : LMI_SoftwareIdentity(env),
        "LMI_SystemSoftwareCollection"    : LMI_SystemSoftwareCollection(env),
        "LMI_HostedSoftwareCollection"    : LMI_HostedSoftwareCollection(env),
        "LMI_MemberOfSoftwareCollection"  : LMI_MemberOfSoftwareCollection(env),
        "LMI_InstalledSoftwareIdentity"   : LMI_InstalledSoftwareIdentity(env),
        "LMI_SoftwareIdentityResource"    : LMI_SoftwareIdentityResource(env),
        "LMI_ResourceForSoftwareIdentity" :
                LMI_ResourceForSoftwareIdentity(env),
        "LMI_HostedSoftwareIdentityResource" :
                LMI_HostedSoftwareIdentityResource(env),
        "LMI_SoftwareInstallationService" : \
                LMI_SoftwareInstallationService(env),
        "LMI_SoftwareInstallationServiceCapabilities" : \
                LMI_SoftwareInstallationServiceCapabilities(env),
        "LMI_AssociatedSoftwareInstallationServiceCapabilities" : \
                LMI_AssociatedSoftwareInstallationServiceCapabilities(env),
        "LMI_HostedSoftwareInstallationService" : \
                LMI_HostedSoftwareInstallationService(env),
        "LMI_SoftwareInstallationServiceAffectsElement" : \
                LMI_SoftwareInstallationServiceAffectsElement(env),
        "LMI_SoftwareJob"                 : LMI_SoftwareJob(env),
        "LMI_SoftwareInstallationJob"     : LMI_SoftwareJob(env),
        "LMI_SoftwareVerificationJob"     : LMI_SoftwareJob(env),
        "LMI_SoftwareInstCreation"        : LMI_SoftwareJob(env),
        "LMI_SoftwareInstModification"    : LMI_SoftwareJob(env),
        "LMI_SoftwareMethodResult"        : LMI_SoftwareMethodResult(env),
        "LMI_AffectedSoftwareJobElement"  : LMI_AffectedSoftwareJobElement(env),
        "LMI_AssociatedSoftwareJobMethodResult" : \
                LMI_AssociatedSoftwareJobMethodResult(env),
        "LMI_OwningSoftwareJobElement"    : LMI_OwningSoftwareJobElement(env),
        "LMI_SoftwareIdentityFileCheck"   : LMI_SoftwareIdentityFileCheck(env),
        "LMI_SoftwareIdentityChecks"      : LMI_SoftwareIdentityChecks(env)
    }

    # Initialization of indication manager -- running in separate thread as
    # daemon. That means it does not have to be cleaned up.
    try:
        im = IndicationManager.get_instance(
                env, "Software", util.Configuration.get_instance().namespace,
                queue=Queue())
    except singletonmixin.SingletonException:
        logging.getLogger(__name__).warn(
                'IndicationManager instantiated for second time')
    JobManager.register_filters("LMI_SoftwareInstallationJob", im)
    JobManager.register_filters("LMI_SoftwareVerificationJob", im)

    return providers

def authorize_filter(env, fltr, class_name, op, owner):
    """
    *CIMOM* callback.

    It asks us to verify whether this filter is allowed.

    :param string fltr: Contains the filter that must be authorized.
    :param string class_name: Contains the class name extracted out of the
        filter's ``FROM`` clause.
    :param op: The name of the class for which monitoring is required.
        Only the namespace part is set if *class_name* is a process indication.
    :type op: :py:class:`pywbem.CIMInstanceName`
    :param owner: Destination owner.
    """
    IndicationManager.get_instance().authorize_filter(
            env, fltr, class_name, op, owner)

def activate_filter(env, fltr, class_name, class_path, first_activation):
    """
    *CIMOM* callback.

    It ask us to begin monitoring a resource. The function shall begin
    monitoring the resource according to the filter express only.

    :param string fltr: The filter argument contains the filter specification
        for this subscription to become active.
    :param string class_name: The class name extracted out of the filter's
        ``FROM`` clause.
    :param class_path: The name of the class for which monitoring is required.
        Only the namespace part is set if *class_name* is a process indication.
    :type class_path: :py:class:`pywbem.CIMInstanceName`
    :param boolean first_activation: Set to ``True`` if this is the first
        filter for *class_name*.
    """
    IndicationManager.get_instance().activate_filter(
            env, fltr, class_name, class_path, first_activation)

def deactivate_filter(env, fltr, class_name, class_path, last_activation):
    """
    *CIMOM* callback.

    Informs us that monitoring using this filter should stop.

    :param string fltr: The filter argument containing specification
        for a subscription becoming deactivated.
    :param string class_name: The class name extracted out the filter's
        ``FROM`` clause.
    :param class_path: The name of the class for which monitoring is
        required. Only the namespace part is set if className is a process
        indication.
    :type class_path: :py:class:`pywbem.CIMInstanceName`
    :param boolean last_activation: Set to ``True`` if this is the last filter
        for *class_name*.
    """
    IndicationManager.get_instance().deactivate_filter(
            env, fltr, class_name, class_path, last_activation)

def enable_indications(env):
    """
    *CIMOM* callback.

    Tells us that indications can now be generated. The MB is now prepared
    to process indications. The function is normally called by the MB after
    having done its intialization and processing of persistent subscription
    requests.
    """
    IndicationManager.get_instance().enable_indications(env)

def disable_indications(env):
    """
    *CIMOM* callback.

    Tells us that we should stop generating indications. MB will not accept any
    indications until enabled again. The function is normally called when the
    MB is shutting down indication services either temporarily or permanently.
    """
    IndicationManager.get_instance().disable_indications(env)

def can_unload(_env):
    """
    *CIMOM* callback.

    :returns: Whether providers can be unloaded.
    :rtype: boolean
    """
    return True

def shutdown(_env):
    """
    *CIMOM* callback.

    Release resources upon cleanup.
    """
    if YumDB.isInstantiated():
        YumDB.get_instance().clean_up()
    IndicationManager.get_instance().shutdown()



# -*- encoding: utf-8 -*-
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Roman Rakus <rrakus@redhat.com>
#

from common import AccountBase
from methods import *
import pywbem

class TestService(AccountBase):
    """
    Class for testing LMI_AccountManagementService
    """

    def test_create_account(self):
        """
        Account: Test create account parameters
        """
        # make sure the account will not exist
        clean_account(self.user_name)
        computer_system = self.wbemconnection.ExecQuery('WQL',
            'select * from %s' % self.system_cs_name)[0]
        lams = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_AccountManagementService')[0]

        # create account and test all parameters
        shell = "testshell"
        system_account = True
        dont_create_home = True
        dont_create_group = True
        gid = 0
        gecos = "test create account gecos"
        home_dir = "/test/home"
        password = '$6$9Ky8vI6f$ipRcdc7rgMrtDh.sWOaRSoBck2cLz4eUom8Eze.NaY2DoMmNimuFBrXpJjlPCjMoeFTYC.FdZwj488JZcohyw1'
        uid = 777
        (rc, out) = self.wbemconnection.InvokeMethod("CreateAccount", lams.path,
            Name = self.user_name,
            System = computer_system.path,
            Shell = shell,
            SystemAccount = system_account,
            DontCreateHome = dont_create_home,
            DontCreateGroup = dont_create_group,
            GID = pywbem.Uint32(gid),
            GECOS = gecos,
            HomeDirectory = home_dir,
            Password = password,
            UID = pywbem.Uint32(uid))

        acc = out["Account"]
        idents = out["Identities"]

        # check return values
        # account
        self.assertEqual(rc, 0)
        self.assertEqual(acc["Name"], self.user_name)
        # identities
        for identity in idents:
            if identity["InstanceID"].find("UID") != -1:
                # user identity
                self.assertEqual(identity["InstanceID"], "LMI:UID:%d" %uid)
            else:
                # group identity
                self.assertEqual(identity["InstanceID"], "LMI:GID:%d" %gid)

        # check with system info
        self.assertEqual(field_in_passwd(self.user_name, 2), str(uid))
        self.assertEqual(field_in_passwd(self.user_name, 3), str(gid))
        self.assertEqual(field_in_passwd(self.user_name, 4), gecos)
        self.assertEqual(field_in_passwd(self.user_name, 5), home_dir)
        self.assertEqual(field_in_passwd(self.user_name, 6), shell)
        self.assertEqual(field_in_shadow(self.user_name, 1), password)
        clean_account(self.user_name)

    def test_create_group(self):
        """
        Account: Test create group parameters
        """
        clean_group(self.group_name)
        computer_system = self.wbemconnection.ExecQuery('WQL',
            'select * from %s' % self.system_cs_name)[0]
        lams = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_AccountManagementService')[0]
        system_account = True
        gid = 666
        (rc, out) = self.wbemconnection.InvokeMethod("CreateGroup", lams.path,
            Name=self.group_name,
            System=computer_system.path,
            SystemAccount=system_account,
            GID=pywbem.Uint32(gid))

        group = out["Group"]
        idents = out["Identities"]
        self.assertEqual(rc, 0)
        self.assertEqual(group["Name"], self.group_name)
        for identity in idents:
            self.assertEqual(identity["InstanceID"], "LMI:GID:%d" %gid)

        self.assertEqual(field_in_group(self.group_name, 2), str(gid))
        clean_group(self.group_name)



# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Jan Safranek <jsafrane@redhat.com>
# Authors: Michal Minar <miminar@redhat.com>
# Authors: Roman Rakus <rrakus@redhat.com>
#
"""
Base classes for *OpenLMI Provider* test cases.
"""

import os
import pywbem
import socket
import unittest

from lmi.test import util

def render_iname(iname, indent=2):
    """
    Render object path in human readable way. Result will occupy multiple
    lines. The first line is a class name, which is not indented at all. Other
    lines will be indented with *indent* spaces.

    :param iname: Object path to render.
    :type iname: :py:class:`pywbem.CIMInstanceName`
    :param integer ident: Number of spaces prefixing all lines but the first.
    :returns: *iname* nicely rendered.
    :rtype: string
    """
    if not isinstance(iname, pywbem.CIMInstanceName):
        return repr(iname)
    lines = [ "%s" % iname.classname
            , " "*indent + "namespace: %s" % iname.namespace
            , " "*indent + "keys:"]
    align = max([len(k) for k in iname.keybindings.iterkeys()])
    for key, value in iname.keybindings.iteritems():
        if isinstance(value, pywbem.CIMInstanceName):
            value = render_iname(value, indent + 4)
        lines.append(" "*indent + ("  %%-%ds : %%s" % align) % (key, value))
    return "\n".join(lines)

class BaseLmiTestCase(unittest.TestCase):
    """
    Base class for all LMI test cases.
    """

    #: Value used in ``SystemName`` key properties in various *CIM* instances.
    #: It's also used to fill ``CIM_ComputerySystem.Name`` property.
    SYSTEM_NAME = socket.gethostname()

    @classmethod
    def setUpClass(cls):
        #: Cached value of SystemCreationClassName set with
        #: ``LMI_CS_CLASSNAME`` environment variable.
        cls.system_cs_name = os.environ.get(
                "LMI_CS_CLASSNAME", "PG_ComputerSystem")
        #: *URL* of *CIMOM* we connect to. Overriden with ``LMI_CIMOM_URL``
        #: environment variable.
        cls.url = os.environ.get("LMI_CIMOM_URL", "https://localhost:5989")
        #: User name for authentication with *CIMOM*. Overriden with
        #: ``LMI_CIMOM_USERNAME`` variable.
        cls.username = os.environ.get("LMI_CIMOM_USERNAME", "root")
        #: User's password for authentication with *CIMOM*. Overriden with
        #: ``LMI_CIMOM_PASSWORD`` environment variable.
        cls.password = os.environ.get("LMI_CIMOM_PASSWORD", "")
        #: Name of *CIMOM* we connect to. There are two possible values:
        #: ``"tog-pegasus"`` and ``"sblim-sfcb"``. Overriden with
        #: ``LMI_CIMOM_BROKER`` environment variable.
        cls.cimom = os.environ.get("LMI_CIMOM_BROKER", "tog-pegasus")
        #: Boolean value saying whether to run dangerous tests. These are marked
        #: with :py:func:`mark_dangerous` decorator. This is set with
        #: ``LMI_RUN_DANGEROUS`` environment variable.
        cls.run_dangerous = util.get_environvar('LMI_RUN_DANGEROUS', '0', bool)
        #: Boolean value saying whether to run tedious tests. These are marked
        #: with :py:func:`mark_tedious` decorator. This is set with
        #: ``LMI_RUN_TEDIOUS`` environment variable.
        cls.run_tedious = util.get_environvar('LMI_RUN_TEDIOUS', '1', bool)

    def assertRaisesCIM(self, cim_err_code, func, *args, **kwds):
        """
        This test passes if given function called with supplied arguments
        raises :py:class:`pywbem.CIMError` with given cim error code.
        """
        with self.assertRaises(pywbem.CIMError) as cm:
            func(*args, **kwds)
        self.assertEqual(cim_err_code, cm.exception.args[0])

    def assertCIMNameEqual(self, fst, snd, msg=None):
        """
        Compare two objects of :py:class:`pywbem.CIMInstanceName`. Their host
        properties are not checked.
        """
        if msg is None:
            msg = ( "%s\n\nis not equal to: %s"
                  % (render_iname(fst), render_iname(snd)))
        self.assertTrue(util.check_inames_equal(fst, snd), msg)

    def assertCIMNameIn(self, name, candidates):
        """
        Checks that given :py:class:`pywbem.CIMInstanceName` is present in
        set of candidates. It compares all properties but ``host``.
        """
        for candidate in candidates:
            if util.check_inames_equal(name, candidate):
                return
        self.assertTrue(False, 'name "%s" is not in candidates' % str(name))

    def assertNocaseDictEqual(self, fst, snd, msg=None):
        """
        Compare two no-case dictionaries ignoring the case of their keys.
        """
        fst_dict = {}
        for (key, value) in fst.iteritems():
            fst_dict[key.lower()] = value
        snd_dict = {}
        for (key, value) in snd.iteritems():
            snd_dict[key.lower()] = value
        self.assertEqual(fst_dict, snd_dict, msg)




# Copyright (C) 2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Tomas Bzatek <tbzatek@redhat.com>
#

from journald_common import JournalBase
import pywbem
import unittest

CIM_MESSAGELOG_ITERATOR_RESULT_SUCCESS = 0
CIM_MESSAGELOG_ITERATOR_RESULT_NOT_SUPPORTED = 1
CIM_MESSAGELOG_ITERATOR_RESULT_FAILED = 2

class TestIterators(JournalBase):
    """
    Class for testing LMI_JournalMessageLog iterators
    """

    def test_simple_iteration(self):
        """
        Journal: Test simple iteration and iterator ID stability
        """
        inst = self.wbemconnection.ExecQuery('WQL', 'select * from LMI_JournalMessageLog')[0]
        self.assertTrue(inst is not None)
        (retval, rfirst) = self.wbemconnection.InvokeMethod("PositionToFirstRecord", inst.path)
        self.assertEqual(retval, 0)
        (retval, rget) = self.wbemconnection.InvokeMethod(MethodName="GetRecord", ObjectName=inst.path, PositionToNext=False,
                                                          IterationIdentifier=rfirst['IterationIdentifier'])
        self.assertEqual(retval, 0)
        self.assertEqual(rfirst['IterationIdentifier'], rget['IterationIdentifier'])
        (retval, rget) = self.wbemconnection.InvokeMethod(MethodName="GetRecord", ObjectName=inst.path, PositionToNext=True,
                                                          IterationIdentifier=rget['IterationIdentifier'])
        self.assertEqual(retval, 0)
        self.assertNotEqual(rfirst['IterationIdentifier'], rget['IterationIdentifier'])
        (retval, rclose) = self.wbemconnection.InvokeMethod(MethodName="CancelIteration", ObjectName=inst.path,
                                                            IterationIdentifier=rget['IterationIdentifier'])
        self.assertEqual(retval, 0)


    def test_invalid_cancellation(self):
        """
        Journal: Test invalid IDs cancellation
        """
        inst = self.wbemconnection.ExecQuery('WQL', 'select * from LMI_JournalMessageLog')[0]
        self.assertTrue(inst is not None)
        # Invalid iterator format
        iter_id = "sdfsdfsdgfa"
        self.assertRaisesCIM(pywbem.CIM_ERR_INVALID_PARAMETER,
                             self.wbemconnection.InvokeMethod,
                                 MethodName="CancelIteration", ObjectName=inst.path,
                                 IterationIdentifier=iter_id)
        # Correct format, inactive iterator
        iter_id = "LMI_JournalMessageLog_CMPI_Iter_9878972#0x7f8504021380#s=a13c10bf472048e69dceeee118cd7f84;i=3;b=5b96431d996a416ebf11dbb6543a5084;m=221f7b;t=4dcc1b6e70a7a;x=b1b572ade816b92e"
        (retval, rclose) = self.wbemconnection.InvokeMethod(MethodName="CancelIteration", ObjectName=inst.path,
                                                            IterationIdentifier=iter_id)
        self.assertEqual(retval, CIM_MESSAGELOG_ITERATOR_RESULT_FAILED)


    def test_double_cancellation(self):
        """
        Journal: Test double iterator cancellation
        """
        inst = self.wbemconnection.ExecQuery('WQL', 'select * from LMI_JournalMessageLog')[0]
        self.assertTrue(inst is not None)
        (retval, rfirst) = self.wbemconnection.InvokeMethod("PositionToFirstRecord", inst.path)
        self.assertEqual(retval, 0)
        (retval, _) = self.wbemconnection.InvokeMethod(MethodName="CancelIteration", ObjectName=inst.path,
                                                       IterationIdentifier=rfirst['IterationIdentifier'])
        self.assertEqual(retval, 0)
        (retval, _) = self.wbemconnection.InvokeMethod(MethodName="CancelIteration", ObjectName=inst.path,
                                                       IterationIdentifier=rfirst['IterationIdentifier'])
        self.assertEqual(retval, CIM_MESSAGELOG_ITERATOR_RESULT_FAILED)


    def test_seeking(self):
        """
        Journal: Test seeking back and forth
        """
        inst = self.wbemconnection.ExecQuery('WQL', 'select * from LMI_JournalMessageLog')[0]
        self.assertTrue(inst is not None)
        (retval, rfirst) = self.wbemconnection.InvokeMethod("PositionToFirstRecord", inst.path)
        self.assertEqual(retval, 0)
        (retval, rget1) = self.wbemconnection.InvokeMethod(MethodName="GetRecord", ObjectName=inst.path, PositionToNext=True,
                                                           IterationIdentifier=rfirst['IterationIdentifier'])
        self.assertEqual(retval, 0)
        # Seeking to absolute record index should not be supported
        self.assertRaisesCIM(pywbem.CIM_ERR_INVALID_PARAMETER,
                             self.wbemconnection.InvokeMethod,
                                 MethodName="PositionAtRecord", ObjectName=inst.path, MoveAbsolute=True,
                                 terationIdentifier=rget1['IterationIdentifier'], RecordNumber=pywbem.Sint64(10))
        (retval, rseek) = self.wbemconnection.InvokeMethod(MethodName="PositionAtRecord", ObjectName=inst.path, MoveAbsolute=False,
                                                           IterationIdentifier=rget1['IterationIdentifier'], RecordNumber=pywbem.Sint64(5))
        self.assertEqual(retval, 0)
        (retval, rseek) = self.wbemconnection.InvokeMethod(MethodName="PositionAtRecord", ObjectName=inst.path, MoveAbsolute=False,
                                                           IterationIdentifier=rseek['IterationIdentifier'], RecordNumber=pywbem.Sint64(4))
        self.assertEqual(retval, 0)
        (retval, rseek) = self.wbemconnection.InvokeMethod(MethodName="PositionAtRecord", ObjectName=inst.path, MoveAbsolute=False,
                                                           IterationIdentifier=rseek['IterationIdentifier'], RecordNumber=pywbem.Sint64(-10))
        self.assertEqual(retval, 0)
        self.assertEqual(rfirst['IterationIdentifier'], rseek['IterationIdentifier'])
        (retval, rget2) = self.wbemconnection.InvokeMethod(MethodName="GetRecord", ObjectName=inst.path, PositionToNext=False,
                                                           IterationIdentifier=rseek['IterationIdentifier'])
        self.assertEqual(retval, 0)
        # We should be at the starting position now
        self.assertEqual(rfirst['IterationIdentifier'], rget2['IterationIdentifier'])
        self.assertEqual(rget1['RecordData'], rget2['RecordData'])
        (retval, rclose) = self.wbemconnection.InvokeMethod(MethodName="CancelIteration", ObjectName=inst.path,
                                                            IterationIdentifier=rget2['IterationIdentifier'])
        self.assertEqual(retval, 0)


    def test_stale_iter(self):
        """
        Journal: Test stale iterators
        """
        # Note this test is closely tied to iterator format, keep in sync with the provider code
        inst = self.wbemconnection.ExecQuery('WQL', 'select * from LMI_JournalMessageLog')[0]
        self.assertTrue(inst is not None)
        (retval, rfirst) = self.wbemconnection.InvokeMethod("PositionToFirstRecord", inst.path)
        self.assertEqual(retval, 0)
        iter_pointer = "0x7f8504021380"
        s1 = rfirst['IterationIdentifier'].split('#')
        iter_id = s1[0] + "#" + iter_pointer + "#" + s1[2]
        (retval, rget1) = self.wbemconnection.InvokeMethod(MethodName="GetRecord", ObjectName=inst.path, PositionToNext=False,
                                                           IterationIdentifier=iter_id)
        self.assertEqual(retval, 0)
        self.assertNotEqual(rget1['IterationIdentifier'], iter_id)
        s2 = rget1['IterationIdentifier'].split('#')
        self.assertNotEqual(s1[0], s2[0])
        self.assertNotEqual(s1[1], s2[1])
        self.assertNotEqual(s2[1], iter_pointer)
        self.assertEqual(s1[2], s2[2])
        (retval, rget2) = self.wbemconnection.InvokeMethod(MethodName="GetRecord", ObjectName=inst.path, PositionToNext=False,
                                                           IterationIdentifier=rget1['IterationIdentifier'])
        self.assertEqual(retval, 0)
        self.assertEqual(rget1['IterationIdentifier'], rget2['IterationIdentifier'])
        (retval, rget3) = self.wbemconnection.InvokeMethod(MethodName="GetRecord", ObjectName=inst.path, PositionToNext=True,
                                                           IterationIdentifier=rget2['IterationIdentifier'])
        self.assertEqual(retval, 0)
        self.assertNotEqual(rget2['IterationIdentifier'], rget3['IterationIdentifier'])
        (retval, _) = self.wbemconnection.InvokeMethod(MethodName="CancelIteration", ObjectName=inst.path,
                                                       IterationIdentifier=rget3['IterationIdentifier'])
        self.assertEqual(retval, 0)
        (retval, _) = self.wbemconnection.InvokeMethod(MethodName="CancelIteration", ObjectName=inst.path,
                                                       IterationIdentifier=rfirst['IterationIdentifier'])
        self.assertEqual(retval, 0)

if __name__ == '__main__':
    unittest.main()



# Software Management Providers
#
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

"""Python Provider for LMI_SystemSoftwareCollection

Instruments the CIM class LMI_SystemSoftwareCollection
"""

import pywbem
from pywbem.cim_provider2 import CIMProvider2

from lmi.providers import cmpi_logging
from lmi.software.core import SystemCollection

class LMI_SystemSoftwareCollection(CIMProvider2):
    """Instrument the CIM class LMI_SystemSoftwareCollection

    SystemSoftwareCollection represents the general concept of a collection
    that is scoped (or contained) by a System. It represents a Collection
    that has meaning only in the context of a System, a Collection whose
    elements are restricted by the definition of the System, or both of
    these types of Collections. This meaning is explicitly described by
    the (required) association, HostedCollection.  An example of a
    SystemSoftwareCollection is a Fibre Channel zone that collects network
    ports, port groupings, and aliases (as required by a customer) in the
    context of an AdminDomain. The Collection is not a part of the domain,
    but merely an arbitrary grouping of the devices and other Collections
    in the domain. In other words, the context of the Collection is
    restricted to the domain, and its members are also limited by the
    domain.

    """

    def __init__(self, _env):
        pass

    @cmpi_logging.trace_method
    def get_instance(self, env, model):
        """Return an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstance to be returned.  The
            key properties are set on this instance to correspond to the
            instanceName that was requested.  The properties of the model
            are already filtered according to the PropertyList from the
            request.  Only properties present in the model need to be
            given values.  If you prefer, you can set all of the
            values, and the instance will be filtered for you.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        if not 'InstanceID' in model:
            raise pywbem.CIMError(pywbem.CIM_ERR_INVALID_PARAMETER,
                    "Missing InstanceID key property")
        if model['InstanceID'] != SystemCollection.get_path()['InstanceID']:
            raise pywbem.CIMError(pywbem.CIM_ERR_NOT_FOUND,
                    "No such instance.")

        model['Caption'] = "System RPM Package Collection"
        #model['Description'] = '' # TODO
        #model['ElementName'] = '' # TODO
        return model

    @cmpi_logging.trace_method
    def enum_instances(self, env, model, keys_only):
        """Enumerate instances.

        The WBEM operations EnumerateInstances and EnumerateInstanceNames
        are both mapped to this method.
        This method is a python generator

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        model -- A template of the pywbem.CIMInstances to be generated.
            The properties of the model are already filtered according to
            the PropertyList from the request.  Only properties present in
            the model need to be given values.  If you prefer, you can
            always set all of the values, and the instance will be filtered
            for you.
        keys_only -- A boolean.  True if only the key properties should be
            set on the generated instances.

        Possible Errors:
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        # Prime model.path with knowledge of the keys, so key values on
        # the CIMInstanceName (model.path) will automatically be set when
        # we set property values on the model.
        model.path.update({'InstanceID': None})

        model['InstanceID'] = SystemCollection.get_path()["InstanceID"]
        if keys_only is False:
            yield self.get_instance(env, model)
        else:
            yield model

    @cmpi_logging.trace_method
    def set_instance(self, env, instance, modify_existing):
        """Return a newly created or modified instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance -- The new pywbem.CIMInstance.  If modifying an existing
            instance, the properties on this instance have been filtered by
            the PropertyList from the request.
        modify_existing -- True if ModifyInstance, False if CreateInstance

        Return the new instance.  The keys must be set on the new instance.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_ALREADY_EXISTS (the CIM Instance already exists -- only
            valid if modify_existing is False, indicating that the operation
            was CreateInstance)
        CIM_ERR_NOT_FOUND (the CIM Instance does not exist -- only valid
            if modify_existing is True, indicating that the operation
            was ModifyInstance)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)

    @cmpi_logging.trace_method
    def delete_instance(self, env, instance_name):
        """Delete an instance.

        Keyword arguments:
        env -- Provider Environment (pycimmb.ProviderEnvironment)
        instance_name -- A pywbem.CIMInstanceName specifying the instance
            to delete.

        Possible Errors:
        CIM_ERR_ACCESS_DENIED
        CIM_ERR_NOT_SUPPORTED
        CIM_ERR_INVALID_NAMESPACE
        CIM_ERR_INVALID_PARAMETER (including missing, duplicate, unrecognized
            or otherwise incorrect parameters)
        CIM_ERR_INVALID_CLASS (the CIM Class does not exist in the specified
            namespace)
        CIM_ERR_NOT_FOUND (the CIM Class does exist, but the requested CIM
            Instance does not exist in the specified namespace)
        CIM_ERR_FAILED (some other unspecified error occurred)

        """
        raise pywbem.CIMError(pywbem.CIM_ERR_NOT_SUPPORTED)




# -*- encoding: utf-8 -*-
# Copyright (C) 2012-2013 Red Hat, Inc.  All rights reserved.
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
#
# Authors: Roman Rakus <rrakus@redhat.com>
#

from common import AccountBase
from methods import *

class TestGroup(AccountBase):
    """
    Class for testing LMI_Group class
    """
    def test_group_properties(self):
        """
        Account: Test if there are key and main properties in LMI_Group
        """
        slct = "select * from LMI_Group"
        instances = self.wbemconnection.ExecQuery('WQL', slct)
        self.assertTrue(len(instances) > 0)
        # check if it provides key properties
        for attr in ["CreationClassName", "Name"]:
            self.assertIsNotNone(instances[0][attr])

        # check if it provides other properties, which should be set
        for attr in ["ElementName", "InstanceID"]:
            self.assertIsNotNone(instances[0][attr])

    def test_create_group(self):
        """
        Account: Test to create group
        """
        # make sure the group will not exist
        clean_group(self.group_name)
        computer_system = self.wbemconnection.ExecQuery('WQL',
            'select * from %s' % self.system_cs_name)[0]
        lams = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_AccountManagementService')[0]
        self.wbemconnection.InvokeMethod("CreateGroup", lams.path,
            Name=self.group_name, System=computer_system.path)
        # The group now should be created, check it
        self.assertEqual(field_in_group(self.group_name, 0), self.group_name)
        # now delete that group
        clean_group(self.group_name)

    def test_delete_group(self):
        """
        Account: Test to delete group
        """
        # make sure the group will exist
        create_group(self.group_name)
        i = self.wbemconnection.ExecQuery('WQL',
            'select * from LMI_Group where Name = "%s"' % self.group_name)[0]
        self.wbemconnection.DeleteInstance(i.path)
        # check if it was really deleted
        self.assertIsNone(field_in_group(self.group_name, 0))
        clean_group(self.group_name)




#pylint: disable-all
"""
A Python Singleton mixin class that makes use of some of the ideas
found at http://c2.com/cgi/wiki?PythonSingleton. Just inherit
from it and you have a singleton. No code is required in
subclasses to create singleton behavior -- inheritance from
Singleton is all that is needed.

Singleton creation is threadsafe.

USAGE:

Just inherit from Singleton. If you need a constructor, include
an __init__() method in your class as you usually would. However,
if your class is S, you instantiate the singleton using S.get_instance()
instead of S(). Repeated calls to S.get_instance() return the
originally-created instance.

For example:

class S(Singleton):

    def __init__(self, a, b=1):
        pass

S1 = S.get_instance(1, b=3)


Most of the time, that's all you need to know. However, there are some
other useful behaviors. Read on for a full description:

1) Getting the singleton:

    S.get_instance()

returns the instance of S. If none exists, it is created.

2) The usual idiom to construct an instance by calling the class, i.e.

    S()

is disabled for the sake of clarity.

For one thing, the S() syntax means instantiation, but get_instance()
usually does not cause instantiation. So the S() syntax would
be misleading.

Because of that, if S() were allowed, a programmer who didn't
happen to notice the inheritance from Singleton (or who
wasn't fully aware of what a Singleton pattern
does) might think he was creating a new instance,
which could lead to very unexpected behavior.

So, overall, it is felt that it is better to make things clearer
by requiring the call of a class method that is defined in
Singleton. An attempt to instantiate via S() will result
in a SingletonException being raised.

3) Use __S.__init__() for instantiation processing,
since S.get_instance() runs S.__init__(), passing it the args it has received.

If no data needs to be passed in at instantiation time,
you don't need S.__init__().

4) If S.__init__(.) requires parameters, include them ONLY in the
first call to S.get_instance(). If subsequent calls have arguments,
a SingletonException is raised by default.

If you find it more convenient for subsequent calls to be allowed to
have arguments, but for those argumentsto be ignored, just include
'ignoreSubsequent = True' in your class definition, i.e.:

  class S(Singleton):

      ignoreSubsequent = True

      def __init__(self, a, b=1):
          pass

5) For testing, it is sometimes convenient for all existing singleton
instances to be forgotten, so that new instantiations can occur. For that
reason, a _forget_all_singletons() function is included. Just call

  _forget_all_singletons()

and it is as if no earlier instantiations have occurred.

6) As an implementation detail, classes that inherit
from Singleton may not have their own __new__
methods. To make sure this requirement is followed,
an exception is raised if a Singleton subclass includ
es __new__. This happens at subclass instantiation
time (by means of the MetaSingleton metaclass.


By Gary Robinson, grobinson@flyfi.com. No rights reserved --
placed in the public domain -- which is only reasonable considering
how much it owes to other people's code and ideas which are in the
public domain. The idea of using a metaclass came from
a  comment on Gary's blog (see
http://www.garyrobinson.net/2004/03/python_singleto.html#comments).
Other improvements came from comments and email from other
people who saw it online. (See the blog post and comments
for further credits.)

Not guaranteed to be fit for any particular purpose. Use at your
own risk.
"""

import threading

class SingletonException(Exception):
    """
    Base exception related to singleton handling.
    """
    pass

_ST_SINGLETONS = set()
_LOCK_FOR_SINGLETONS = threading.RLock()
# Ensure only one instance of each Singleton class is created. This is not
# bound to the _LOCK_FOR_SINGLETON_CREATION = threading.RLock() individual
# Singleton class since we need to ensure that there is only one mutex for each
# Singleton class, which would require having a lock when setting up the
# Singleton class, which is what this is anyway.  So, when any Singleton is
# created, we lock this lock and then we don't need to lock it again for that
# class.
_LOCK_FOR_SINGLETON_CREATION = threading.RLock()

def _create_singleton_instance(cls, lst_args, dct_kw_args):
    """
    Creates singleton instance and stores its class in set.
    """
    _LOCK_FOR_SINGLETON_CREATION.acquire()
    try:
        if cls._is_instantiated(): # some other thread got here first
            return

        instance = cls.__new__(cls)
        try:
            instance.__init__(*lst_args, **dct_kw_args)
        except TypeError, exc:
            if '__init__() takes' in exc.message:
                raise SingletonException, (
                    'If the singleton requires __init__ args,'
                    ' supply them on first call to get_instance().')
            else:
                raise
        cls.c_instance = instance
        _add_singleton(cls)
    finally:
        _LOCK_FOR_SINGLETON_CREATION.release()

def _add_singleton(cls):
    """
    Adds class to singleton set.
    """
    _LOCK_FOR_SINGLETONS.acquire()
    try:
        assert cls not in _ST_SINGLETONS
        _ST_SINGLETONS.add(cls)
    finally:
        _LOCK_FOR_SINGLETONS.release()

def _remove_singleton(cls):
    """
    Removes class from singleton set.
    """
    _LOCK_FOR_SINGLETONS.acquire()
    try:
        if cls in _ST_SINGLETONS:
            _ST_SINGLETONS.remove(cls)
    finally:
        _LOCK_FOR_SINGLETONS.release()

def _forget_all_singletons():
    '''
    This is useful in tests, since it is hard to know which singletons need
    to be cleared to make a test work.
    '''
    _LOCK_FOR_SINGLETONS.acquire()
    try:
        for cls in _ST_SINGLETONS.copy():
            cls._forget_class_instance_reference_for_testing()

        # Might have created some Singletons in the process of tearing down.
        # Try one more time - there should be a limit to this.
        i_num_singletons = len(_ST_SINGLETONS)
        if len(_ST_SINGLETONS) > 0:
            for cls in _ST_SINGLETONS.copy():
                cls._forget_class_instance_reference_for_testing()
                i_num_singletons -= 1
                assert i_num_singletons == len(_ST_SINGLETONS), \
                        'Added a singleton while destroying ' + str(cls)
        assert len(_ST_SINGLETONS) == 0, _ST_SINGLETONS
    finally:
        _LOCK_FOR_SINGLETONS.release()

class MetaSingleton(type):
    """
    Metaclass for Singleton base class.
    """
    def __new__(mcs, str_name, tup_bases, dct):
        if dct.has_key('__new__'):
            raise SingletonException, 'Can not override __new__ in a Singleton'
        return super(MetaSingleton, mcs).__new__(
                mcs, str_name, tup_bases, dct)

    def __call__(cls, *lst_args, **dictArgs):
        raise SingletonException, \
                'Singletons may only be instantiated through get_instance()'

class Singleton(object):
    """
    Base class for all singletons.
    """
    __metaclass__ = MetaSingleton

    def get_instance(cls, *lst_args, **dct_kw_args):
        """
        Call this to instantiate an instance or retrieve the existing instance.
        If the singleton requires args to be instantiated, include them the first
        time you call get_instance.
        """
        if cls._is_instantiated():
            if (   (lst_args or dct_kw_args)
               and not hasattr(cls, 'ignoreSubsequent')):
                raise SingletonException, (
                        'Singleton already instantiated, but get_instance()'
                        ' called with args.')
        else:
            _create_singleton_instance(cls, lst_args, dct_kw_args)

        return cls.c_instance   #pylint: disable=E1101
    get_instance = classmethod(get_instance)

    def _is_instantiated(cls):
        """
        Don't use hasattr(cls, 'c_instance'), because that screws things
        up if there is a singleton that extends another singleton.
        hasattr looks in the base class if it doesn't find in subclass.
        """
        return 'c_instance' in cls.__dict__
    _is_instantiated = classmethod(_is_instantiated)

    # This can be handy for public use also
    isInstantiated = _is_instantiated

    def _forget_class_instance_reference_for_testing(cls):
        """
        This is designed for convenience in testing -- sometimes you
        want to get rid of a singleton during test code to see what
        happens when you call get_instance() under a new situation.

        To really delete the object, all external references to it
        also need to be deleted.
        """
        try:
            if hasattr(cls.c_instance, '_prepare_to_forget_singleton'):
                # tell instance to release anything it might be holding onto.
                cls.c_instance._prepare_to_forget_singleton()
            del cls.c_instance
            _remove_singleton(cls)
        except AttributeError:
            # run up the chain of base classes until we find the one that has
            # the instance and then delete it there
            for base_class in cls.__bases__:
                if issubclass(base_class, Singleton):
                    base_class._forget_class_instance_reference_for_testing()
    _forget_class_instance_reference_for_testing = classmethod(
        _forget_class_instance_reference_for_testing)


if __name__ == '__main__':

    import unittest
    import time

    class SingletonMixinPublicTestCase(unittest.TestCase):
        """
        TestCase for singleton class.
        """
        def testReturnsSameObject(self):    #pylint: disable=C0103
            """
            Demonstrates normal use -- just call get_instance and it returns a singleton instance
            """

            class Foo(Singleton):
                """Singleton child class."""
                def __init__(self):
                    super(Foo, self).__init__()

            a1 = Foo.get_instance()
            a2 = Foo.get_instance()
            self.assertEquals(id(a1), id(a2))

        def testInstantiateWithMultiArgConstructor(self):#pylint: disable=C0103
            """
            If the singleton needs args to construct, include them in the first
            call to get instances.
            """

            class Bar(Singleton):
                """Singleton child class."""

                def __init__(self, arg1, arg2):
                    super(Bar, self).__init__()
                    self.arg1 = arg1
                    self.arg2 = arg2

            b1 = Bar.get_instance('arg1 value', 'arg2 value')
            b2 = Bar.get_instance()
            self.assertEquals(b1.arg1, 'arg1 value')
            self.assertEquals(b1.arg2, 'arg2 value')
            self.assertEquals(id(b1), id(b2))

        def testInstantiateWithKeywordArg(self):
            """
            Test instantiation with keyword arguments.
            """

            class Baz(Singleton):
                """Singleton child class."""
                def __init__(self, arg1=5):
                    super(Baz, self).__init__()
                    self.arg1 = arg1

            b1 = Baz.get_instance('arg1 value')
            b2 = Baz.get_instance()
            self.assertEquals(b1.arg1, 'arg1 value')
            self.assertEquals(id(b1), id(b2))

        def testTryToInstantiateWithoutNeededArgs(self):
            """
            This tests, improper instantiation.
            """

            class Foo(Singleton):
                """Singleton child class."""
                def __init__(self, arg1, arg2):
                    super(Foo, self).__init__()
                    self.arg1 = arg1
                    self.arg2 = arg2

            self.assertRaises(SingletonException, Foo.get_instance)

        def testPassTypeErrorIfAllArgsThere(self):
            """
            Make sure the test for capturing missing args doesn't interfere
            with a normal TypeError.
            """
            class Bar(Singleton):
                """Singleton child class."""
                def __init__(self, arg1, arg2):
                    super(Bar, self).__init__()
                    self.arg1 = arg1
                    self.arg2 = arg2
                    raise TypeError, 'some type error'

            self.assertRaises(TypeError, Bar.get_instance, 1, 2)

        def testTryToInstantiateWithoutGetInstance(self):
            """
            Demonstrates that singletons can ONLY be instantiated through
            get_instance, as long as they call Singleton.__init__ during
            construction.

            If this check is not required, you don't need to call
            Singleton.__init__().
            """

            class A(Singleton):
                def __init__(self):
                    super(A, self).__init__()

            self.assertRaises(SingletonException, A)

        def testDontAllowNew(self):

            def instantiatedAnIllegalClass():
                class A(Singleton):
                    def __init__(self):
                        super(A, self).__init__()

                    def __new__(metaclass, str_name, tup_bases, dct):
                        return super(MetaSingleton, metaclass).__new__(
                                metaclass, str_name, tup_bases, dct)

            self.assertRaises(SingletonException, instantiatedAnIllegalClass)


        def testDontAllowArgsAfterConstruction(self):
            class B(Singleton):

                def __init__(self, arg1, arg2):
                    super(B, self).__init__()
                    self.arg1 = arg1
                    self.arg2 = arg2

            B.get_instance('arg1 value', 'arg2 value')
            self.assertRaises(SingletonException, B, 'arg1 value', 'arg2 value')

        def test_forgetClassInstanceReferenceForTesting(self):
            class A(Singleton):
                def __init__(self):
                    super(A, self).__init__()
            class B(A):
                def __init__(self):
                    super(B, self).__init__()

            # check that changing the class after forgetting the instance
            # produces an instance of the new class
            a = A.get_instance()
            assert a.__class__.__name__ == 'A'
            A._forget_class_instance_reference_for_testing()
            b = B.get_instance()
            assert b.__class__.__name__ == 'B'

            # check that invoking the 'forget' on a subclass still deletes
            # the instance
            B._forget_class_instance_reference_for_testing()
            a = A.get_instance()
            B._forget_class_instance_reference_for_testing()
            b = B.get_instance()
            assert b.__class__.__name__ == 'B'

        def test_forgetAllSingletons(self):
            # Should work if there are no singletons
            _forget_all_singletons()

            class A(Singleton):
                ciInitCount = 0
                def __init__(self):
                    super(A, self).__init__()
                    A.ciInitCount += 1

            A.get_instance()
            self.assertEqual(A.ciInitCount, 1)

            A.get_instance()
            self.assertEqual(A.ciInitCount, 1)

            _forget_all_singletons()
            A.get_instance()
            self.assertEqual(A.ciInitCount, 2)

        def test_threadedCreation(self):
            # Check that only one Singleton is created even if multiple threads
            # try at the same time. If fails, would see assert in _add_singleton
            class Test_Singleton(Singleton):
                def __init__(self):
                    super(Test_Singleton, self).__init__()

            class Test_SingletonThread(threading.Thread):
                def __init__(self, fTargetTime):
                    super(Test_SingletonThread, self).__init__()
                    self._fTargetTime = fTargetTime
                    self._eException = None

                def run(self):
                    try:
                        fSleepTime =  self._fTargetTime - time.time()
                        if fSleepTime > 0:
                            time.sleep(fSleepTime)
                        Test_Singleton.get_instance()
                    except Exception, exc:
                        self._eException = exc

            fTargetTime = time.time() + 0.1
            lstThreads = []
            for _ in xrange(100):
                t = Test_SingletonThread(fTargetTime)
                t.start()
                lstThreads.append(t)
            eException = None
            for t in lstThreads:
                t.join()
                if t._eException and not eException:
                    eException = t._eException
            if eException:
                raise eException

        def testNoInit(self):
            """
            Demonstrates use with a class not defining __init__
            """

            class A(Singleton):
                pass

                #INTENTIONALLY UNDEFINED:
                #def __init__(self):
                #    super(A, self).__init__()

            A.get_instance() #Make sure no exception is raised

        def testMultipleGetInstancesWithArgs(self):

            class A(Singleton):

                ignoreSubsequent = True

                def __init__(self, a, b=1):
                    pass

            a1 = A.get_instance(1)
            # ignores the second call because of ignoreSubsequent
            a2 = A.get_instance(2)

            class B(Singleton):

                def __init__(self, a, b=1):
                    pass

            b1 = B.get_instance(1)
            # No ignoreSubsequent included
            self.assertRaises(SingletonException, B.get_instance, 2)

            class C(Singleton):

                def __init__(self, a=1):
                    pass

            c1 = C.get_instance(a=1)
            # No ignoreSubsequent included
            self.assertRaises(SingletonException, C.get_instance, a=2)

        def testInheritance(self):
            """
            It's sometimes said that you can't subclass a singleton (see, for instance,
            http://steve.yegge.googlepages.com/singleton-considered-stupid point e). This
            test shows that at least rudimentary subclassing works fine for us.
            """

            class A(Singleton):

                def set_x(self, x):
                    self.x = x

                def setZ(self, z):
                    raise NotImplementedError

            class B(A):

                def set_x(self, x):
                    self.x = -x

                def set_y(self, y):
                    self.y = y

            a = A.get_instance()
            a.set_x(5)
            b = B.get_instance()
            b.set_x(5)
            b.set_y(50)
            self.assertEqual((a.x, b.x, b.y), (5, -5, 50))
            self.assertRaises(AttributeError, eval, 'a.set_y', {}, locals())
            self.assertRaises(NotImplementedError, b.setZ, 500)

    unittest.main()

